@@ -20,6 +20,7 @@
 package boot_test
 
 import (
+	"io/ioutil"
 	"path/filepath"
 	"testing"
 
@@ -38,27 +39,6 @@ import (
 
 func TestBoot(t *testing.T) { TestingT(t) }
 
-type kernelOSSuite struct {
-	testutil.BaseTest
-	bootloader *boottest.MockBootloader
-}
-
-var _ = Suite(&kernelOSSuite{})
-
-func (s *kernelOSSuite) SetUpTest(c *C) {
-	s.BaseTest.SetUpTest(c)
-	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
-	dirs.SetRootDir(c.MkDir())
-	s.bootloader = boottest.NewMockBootloader("mock", c.MkDir())
-	bootloader.Force(s.bootloader)
-}
-
-func (s *kernelOSSuite) TearDownTest(c *C) {
-	s.BaseTest.TearDownTest(c)
-	dirs.SetRootDir("")
-	bootloader.Force(nil)
-}
-
 const packageKernel = `
 name: ubuntu-kernel
 version: 4.0-1
@@ -66,123 +46,47 @@ type: kernel
 vendor: Someone
 `
 
-func (s *kernelOSSuite) TestExtractKernelAssetsAndRemove(c *C) {
-	files := [][]string{
-		{"kernel.img", "I'm a kernel"},
-		{"initrd.img", "...and I'm an initrd"},
-		{"dtbs/foo.dtb", "g'day, I'm foo.dtb"},
-		{"dtbs/bar.dtb", "hello, I'm bar.dtb"},
-		// must be last
-		{"meta/kernel.yaml", "version: 4.2"},
-	}
+// baseKernelOSSuite is used to setup the common test environment
+type baseKernelOSSuite struct {
+	testutil.BaseTest
 
-	si := &snap.SideInfo{
-		RealName: "ubuntu-kernel",
-		Revision: snap.R(42),
-	}
-	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
-	snapf, err := snap.Open(fn)
-	c.Assert(err, IsNil)
-
-	info, err := snap.ReadInfoFromSnapFile(snapf, si)
-	c.Assert(err, IsNil)
-
-	err = boot.ExtractKernelAssets(info, snapf)
-	c.Assert(err, IsNil)
-
-	// this is where the kernel/initrd is unpacked
-	bootdir := s.bootloader.Dir()
-
-	kernelAssetsDir := filepath.Join(bootdir, "ubuntu-kernel_42.snap")
-
-	for _, def := range files {
-		if def[0] == "meta/kernel.yaml" {
-			break
-		}
-
-		fullFn := filepath.Join(kernelAssetsDir, def[0])
-		c.Check(fullFn, testutil.FileEquals, def[1])
-	}
-
-	// remove
-	err = boot.RemoveKernelAssets(info)
-	c.Assert(err, IsNil)
-
-	c.Check(osutil.FileExists(kernelAssetsDir), Equals, false)
+	bootdir string
 }
 
-func (s *kernelOSSuite) TestExtractKernelAssetsNoUnpacksKernelForGrub(c *C) {
-	// pretend to be a grub system
-	mockGrub := boottest.NewMockBootloader("grub", c.MkDir())
-	bootloader.Force(mockGrub)
+func (s *baseKernelOSSuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
 
-	files := [][]string{
-		{"kernel.img", "I'm a kernel"},
-		{"initrd.img", "...and I'm an initrd"},
-		{"meta/kernel.yaml", "version: 4.2"},
-	}
-	si := &snap.SideInfo{
-		RealName: "ubuntu-kernel",
-		Revision: snap.R(42),
-	}
-	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
-	snapf, err := snap.Open(fn)
-	c.Assert(err, IsNil)
+	dirs.SetRootDir(c.MkDir())
+	s.AddCleanup(func() { dirs.SetRootDir("") })
+	restore := snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {})
+	s.AddCleanup(restore)
+	restore = release.MockOnClassic(false)
+	s.AddCleanup(restore)
 
-	info, err := snap.ReadInfoFromSnapFile(snapf, si)
-	c.Assert(err, IsNil)
-
-	err = boot.ExtractKernelAssets(info, snapf)
-	c.Assert(err, IsNil)
-
-	// kernel is *not* here
-	kernimg := filepath.Join(mockGrub.Dir(), "ubuntu-kernel_42.snap", "kernel.img")
-	c.Assert(osutil.FileExists(kernimg), Equals, false)
+	s.bootdir = filepath.Join(dirs.GlobalRootDir, "boot")
 }
 
-func (s *kernelOSSuite) TestExtractKernelForceWorks(c *C) {
-	// pretend to be a grub system
-	mockGrub := boottest.NewMockBootloader("grub", c.MkDir())
-	bootloader.Force(mockGrub)
+// kernelOSSuite tests the abstract bootloader behaviour including
+// bootenv setting, error handling etc
+type kernelOSSuite struct {
+	baseKernelOSSuite
 
-	files := [][]string{
-		{"kernel.img", "I'm a kernel"},
-		{"initrd.img", "...and I'm an initrd"},
-		{"meta/force-kernel-extraction", ""},
-		{"meta/kernel.yaml", "version: 4.2"},
-	}
-	si := &snap.SideInfo{
-		RealName: "ubuntu-kernel",
-		Revision: snap.R(42),
-	}
-	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
-	snapf, err := snap.Open(fn)
-	c.Assert(err, IsNil)
+	loader *boottest.MockBootloader
+}
 
-	info, err := snap.ReadInfoFromSnapFile(snapf, si)
-	c.Assert(err, IsNil)
+var _ = Suite(&kernelOSSuite{})
 
-	err = boot.ExtractKernelAssets(info, snapf)
-	c.Assert(err, IsNil)
+func (s *kernelOSSuite) SetUpTest(c *C) {
+	s.baseKernelOSSuite.SetUpTest(c)
 
-	// kernel is extracted
-	kernimg := filepath.Join(mockGrub.Dir(), "ubuntu-kernel_42.snap", "kernel.img")
-	c.Assert(osutil.FileExists(kernimg), Equals, true)
-	// initrd
-	initrdimg := filepath.Join(mockGrub.Dir(), "ubuntu-kernel_42.snap", "initrd.img")
-	c.Assert(osutil.FileExists(initrdimg), Equals, true)
-
-	// ensure that removal of assets also works
-	err = boot.RemoveKernelAssets(info)
-	c.Assert(err, IsNil)
-	exists, _, err := osutil.DirExists(filepath.Dir(kernimg))
-	c.Assert(err, IsNil)
-	c.Check(exists, Equals, false)
+	s.loader = boottest.NewMockBootloader("mock", c.MkDir())
+	bootloader.Force(s.loader)
+	s.AddCleanup(func() { bootloader.Force(nil) })
 }
 
 func (s *kernelOSSuite) TestExtractKernelAssetsError(c *C) {
 	info := &snap.Info{}
-	info.Type = snap.TypeApp
+	info.SnapType = snap.TypeApp
 
 	err := boot.ExtractKernelAssets(info, nil)
 	c.Assert(err, ErrorMatches, `cannot extract kernel assets from snap type "app"`)
@@ -198,22 +102,21 @@ func (s *kernelOSSuite) TestSetNextBootOnClassic(c *C) {
 	err := boot.SetNextBoot(snapInfo)
 	c.Assert(err, ErrorMatches, "cannot set next boot on classic systems")
 
-	c.Assert(s.bootloader.BootVars, HasLen, 0)
+	c.Assert(s.loader.BootVars, HasLen, 0)
 }
 
 func (s *kernelOSSuite) TestSetNextBootForCore(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	info := &snap.Info{}
-	info.Type = snap.TypeOS
+	info.SnapType = snap.TypeOS
 	info.RealName = "core"
 	info.Revision = snap.R(100)
 
 	err := boot.SetNextBoot(info)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.bootloader.BootVars, DeepEquals, map[string]string{
+	v, err := s.loader.GetBootVars("snap_try_core", "snap_mode")
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, map[string]string{
 		"snap_try_core": "core_100.snap",
 		"snap_mode":     "try",
 	})
@@ -222,18 +125,17 @@ func (s *kernelOSSuite) TestSetNextBootForCore(c *C) {
 }
 
 func (s *kernelOSSuite) TestSetNextBootWithBaseForCore(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	info := &snap.Info{}
-	info.Type = snap.TypeBase
+	info.SnapType = snap.TypeBase
 	info.RealName = "core18"
 	info.Revision = snap.R(1818)
 
 	err := boot.SetNextBoot(info)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.bootloader.BootVars, DeepEquals, map[string]string{
+	v, err := s.loader.GetBootVars("snap_try_core", "snap_mode")
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, map[string]string{
 		"snap_try_core": "core18_1818.snap",
 		"snap_mode":     "try",
 	})
@@ -242,67 +144,70 @@ func (s *kernelOSSuite) TestSetNextBootWithBaseForCore(c *C) {
 }
 
 func (s *kernelOSSuite) TestSetNextBootForKernel(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	info := &snap.Info{}
-	info.Type = snap.TypeKernel
+	info.SnapType = snap.TypeKernel
 	info.RealName = "krnl"
 	info.Revision = snap.R(42)
 
 	err := boot.SetNextBoot(info)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.bootloader.BootVars, DeepEquals, map[string]string{
+	v, err := s.loader.GetBootVars("snap_try_kernel", "snap_mode")
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, map[string]string{
 		"snap_try_kernel": "krnl_42.snap",
 		"snap_mode":       "try",
 	})
 
-	s.bootloader.BootVars["snap_kernel"] = "krnl_40.snap"
-	s.bootloader.BootVars["snap_try_kernel"] = "krnl_42.snap"
+	bootVars := map[string]string{
+		"snap_kernel":     "krnl_40.snap",
+		"snap_try_kernel": "krnl_42.snap"}
+	s.loader.SetBootVars(bootVars)
 	c.Check(boot.ChangeRequiresReboot(info), Equals, true)
 
 	// simulate good boot
-	s.bootloader.BootVars["snap_kernel"] = "krnl_42.snap"
+	bootVars = map[string]string{"snap_kernel": "krnl_42.snap"}
+	s.loader.SetBootVars(bootVars)
 	c.Check(boot.ChangeRequiresReboot(info), Equals, false)
 }
 
 func (s *kernelOSSuite) TestSetNextBootForKernelForTheSameKernel(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	info := &snap.Info{}
-	info.Type = snap.TypeKernel
+	info.SnapType = snap.TypeKernel
 	info.RealName = "krnl"
 	info.Revision = snap.R(40)
 
-	s.bootloader.BootVars["snap_kernel"] = "krnl_40.snap"
+	bootVars := map[string]string{"snap_kernel": "krnl_40.snap"}
+	s.loader.SetBootVars(bootVars)
 
 	err := boot.SetNextBoot(info)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.bootloader.BootVars, DeepEquals, map[string]string{
+	v, err := s.loader.GetBootVars("snap_kernel")
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, map[string]string{
 		"snap_kernel": "krnl_40.snap",
 	})
 }
 
 func (s *kernelOSSuite) TestSetNextBootForKernelForTheSameKernelTryMode(c *C) {
-	restore := release.MockOnClassic(false)
-	defer restore()
-
 	info := &snap.Info{}
-	info.Type = snap.TypeKernel
+	info.SnapType = snap.TypeKernel
 	info.RealName = "krnl"
 	info.Revision = snap.R(40)
 
-	s.bootloader.BootVars["snap_kernel"] = "krnl_40.snap"
-	s.bootloader.BootVars["snap_try_kernel"] = "krnl_99.snap"
-	s.bootloader.BootVars["snap_mode"] = "try"
+	bootVars := map[string]string{
+		"snap_kernel":     "krnl_40.snap",
+		"snap_try_kernel": "krnl_99.snap",
+		"snap_mode":       "try"}
+	s.loader.SetBootVars(bootVars)
 
 	err := boot.SetNextBoot(info)
 	c.Assert(err, IsNil)
 
-	c.Assert(s.bootloader.BootVars, DeepEquals, map[string]string{
+	v, err := s.loader.GetBootVars("snap_kernel", "snap_try_kernel", "snap_mode")
+	c.Assert(err, IsNil)
+	c.Assert(v, DeepEquals, map[string]string{
 		"snap_kernel":     "krnl_40.snap",
 		"snap_try_kernel": "",
 		"snap_mode":       "",
@@ -330,7 +235,249 @@ func (s *kernelOSSuite) TestInUse(c *C) {
 		{"snap_kernel", "kernel_111.snap", "kernel", snap.R(1), false},
 		{"snap_try_kernel", "kernel_111.snap", "kernel", snap.R(1), false},
 	} {
-		s.bootloader.BootVars[t.bootVarKey] = t.bootVarValue
+		s.loader.BootVars[t.bootVarKey] = t.bootVarValue
 		c.Assert(boot.InUse(t.snapName, t.snapRev), Equals, t.inUse, Commentf("unexpected result: %s %s %v", t.snapName, t.snapRev, t.inUse))
 	}
 }
+
+func (s *kernelOSSuite) TestNameAndRevnoFromSnapValid(c *C) {
+	info, err := boot.NameAndRevnoFromSnap("foo_2.snap")
+	c.Assert(err, IsNil)
+	c.Assert(info.Name, Equals, "foo")
+	c.Assert(info.Revision, Equals, snap.R(2))
+}
+
+func (s *kernelOSSuite) TestNameAndRevnoFromSnapInvalidFormat(c *C) {
+	_, err := boot.NameAndRevnoFromSnap("invalid")
+	c.Assert(err, ErrorMatches, `input "invalid" has invalid format \(not enough '_'\)`)
+}
+
+func BenchmarkNameAndRevno(b *testing.B) {
+	for n := 0; n < b.N; n++ {
+		for _, sn := range []string{
+			"core_21.snap",
+			"kernel_41.snap",
+			"some-long-kernel-name-kernel_82.snap",
+			"what-is-this-core_111.snap",
+		} {
+			boot.NameAndRevnoFromSnap(sn)
+		}
+	}
+}
+
+func (s *kernelOSSuite) TestCurrentBootNameAndRevision(c *C) {
+	s.loader.BootVars["snap_core"] = "core_2.snap"
+	s.loader.BootVars["snap_kernel"] = "canonical-pc-linux_2.snap"
+
+	current, err := boot.GetCurrentBoot(snap.TypeOS)
+	c.Check(err, IsNil)
+	c.Check(current.Name, Equals, "core")
+	c.Check(current.Revision, Equals, snap.R(2))
+
+	current, err = boot.GetCurrentBoot(snap.TypeKernel)
+	c.Check(err, IsNil)
+	c.Check(current.Name, Equals, "canonical-pc-linux")
+	c.Check(current.Revision, Equals, snap.R(2))
+
+	s.loader.BootVars["snap_mode"] = "trying"
+	_, err = boot.GetCurrentBoot(snap.TypeKernel)
+	c.Check(err, Equals, boot.ErrBootNameAndRevisionAgain)
+}
+
+func (s *kernelOSSuite) TestCurrentBootNameAndRevisionUnhappy(c *C) {
+	_, err := boot.GetCurrentBoot(snap.TypeKernel)
+	c.Check(err, ErrorMatches, "cannot get name and revision of boot kernel: unset")
+
+	_, err = boot.GetCurrentBoot(snap.TypeOS)
+	c.Check(err, ErrorMatches, "cannot get name and revision of boot snap: unset")
+
+	_, err = boot.GetCurrentBoot(snap.TypeBase)
+	c.Check(err, ErrorMatches, "cannot get name and revision of boot snap: unset")
+
+	_, err = boot.GetCurrentBoot(snap.TypeApp)
+	c.Check(err, ErrorMatches, "internal error: cannot find boot revision for snap type \"app\"")
+}
+
+// ubootKernelOSSuite tests the uboot specific code in the bootloader handling
+type ubootKernelOSSuite struct {
+	baseKernelOSSuite
+}
+
+var _ = Suite(&ubootKernelOSSuite{})
+
+func (s *ubootKernelOSSuite) forceUbootBootloader(c *C) bootloader.Bootloader {
+	mockGadgetDir := c.MkDir()
+	err := ioutil.WriteFile(filepath.Join(mockGadgetDir, "uboot.conf"), nil, 0644)
+	c.Assert(err, IsNil)
+	err = bootloader.InstallBootConfig(mockGadgetDir)
+	c.Assert(err, IsNil)
+
+	loader, err := bootloader.Find()
+	c.Assert(err, IsNil)
+	c.Check(loader, NotNil)
+	bootloader.Force(loader)
+	s.AddCleanup(func() { bootloader.Force(nil) })
+
+	fn := filepath.Join(s.bootdir, "/uboot/uboot.env")
+	c.Assert(osutil.FileExists(fn), Equals, true)
+	return loader
+}
+
+func (s *ubootKernelOSSuite) TestExtractKernelAssetsAndRemoveOnUboot(c *C) {
+	loader := s.forceUbootBootloader(c)
+	c.Assert(loader, NotNil)
+
+	files := [][]string{
+		{"kernel.img", "I'm a kernel"},
+		{"initrd.img", "...and I'm an initrd"},
+		{"dtbs/foo.dtb", "g'day, I'm foo.dtb"},
+		{"dtbs/bar.dtb", "hello, I'm bar.dtb"},
+		// must be last
+		{"meta/kernel.yaml", "version: 4.2"},
+	}
+
+	si := &snap.SideInfo{
+		RealName: "ubuntu-kernel",
+		Revision: snap.R(42),
+	}
+	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
+	snapf, err := snap.Open(fn)
+	c.Assert(err, IsNil)
+
+	info, err := snap.ReadInfoFromSnapFile(snapf, si)
+	c.Assert(err, IsNil)
+
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+
+	// this is where the kernel/initrd is unpacked
+	kernelAssetsDir := filepath.Join(s.bootdir, "/uboot/ubuntu-kernel_42.snap")
+	for _, def := range files {
+		if def[0] == "meta/kernel.yaml" {
+			break
+		}
+
+		fullFn := filepath.Join(kernelAssetsDir, def[0])
+		c.Check(fullFn, testutil.FileEquals, def[1])
+	}
+
+	// it's idempotent
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+
+	// remove
+	err = boot.RemoveKernelAssets(info)
+	c.Assert(err, IsNil)
+	c.Check(osutil.FileExists(kernelAssetsDir), Equals, false)
+
+	// it's idempotent
+	err = boot.RemoveKernelAssets(info)
+	c.Assert(err, IsNil)
+}
+
+// grubKernelOSSuite tests the GRUB specific code in the bootloader handling
+type grubKernelOSSuite struct {
+	baseKernelOSSuite
+}
+
+var _ = Suite(&grubKernelOSSuite{})
+
+func (s *grubKernelOSSuite) forceGrubBootloader(c *C) bootloader.Bootloader {
+	// make mock grub bootenv dir
+	mockGadgetDir := c.MkDir()
+	err := ioutil.WriteFile(filepath.Join(mockGadgetDir, "grub.conf"), nil, 0644)
+	c.Assert(err, IsNil)
+	err = bootloader.InstallBootConfig(mockGadgetDir)
+	c.Assert(err, IsNil)
+
+	loader, err := bootloader.Find()
+	c.Assert(err, IsNil)
+	c.Check(loader, NotNil)
+	loader.SetBootVars(map[string]string{
+		"snap_kernel": "kernel_41.snap",
+		"snap_core":   "core_21.snap",
+	})
+	bootloader.Force(loader)
+	s.AddCleanup(func() { bootloader.Force(nil) })
+
+	fn := filepath.Join(s.bootdir, "/grub/grub.cfg")
+	c.Assert(osutil.FileExists(fn), Equals, true)
+	return loader
+}
+
+func (s *grubKernelOSSuite) TestExtractKernelAssetsNoUnpacksKernelForGrub(c *C) {
+	s.forceGrubBootloader(c)
+
+	files := [][]string{
+		{"kernel.img", "I'm a kernel"},
+		{"initrd.img", "...and I'm an initrd"},
+		{"meta/kernel.yaml", "version: 4.2"},
+	}
+	si := &snap.SideInfo{
+		RealName: "ubuntu-kernel",
+		Revision: snap.R(42),
+	}
+	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
+	snapf, err := snap.Open(fn)
+	c.Assert(err, IsNil)
+
+	info, err := snap.ReadInfoFromSnapFile(snapf, si)
+	c.Assert(err, IsNil)
+
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+
+	// kernel is *not* here
+	kernimg := filepath.Join(s.bootdir, "grub", "ubuntu-kernel_42.snap", "kernel.img")
+	c.Assert(osutil.FileExists(kernimg), Equals, false)
+
+	// it's idempotent
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+}
+
+func (s *grubKernelOSSuite) TestExtractKernelForceWorks(c *C) {
+	s.forceGrubBootloader(c)
+
+	files := [][]string{
+		{"kernel.img", "I'm a kernel"},
+		{"initrd.img", "...and I'm an initrd"},
+		{"meta/force-kernel-extraction", ""},
+		{"meta/kernel.yaml", "version: 4.2"},
+	}
+	si := &snap.SideInfo{
+		RealName: "ubuntu-kernel",
+		Revision: snap.R(42),
+	}
+	fn := snaptest.MakeTestSnapWithFiles(c, packageKernel, files)
+	snapf, err := snap.Open(fn)
+	c.Assert(err, IsNil)
+
+	info, err := snap.ReadInfoFromSnapFile(snapf, si)
+	c.Assert(err, IsNil)
+
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+
+	// kernel is extracted
+	kernimg := filepath.Join(s.bootdir, "/grub/ubuntu-kernel_42.snap/kernel.img")
+	c.Assert(osutil.FileExists(kernimg), Equals, true)
+	// initrd
+	initrdimg := filepath.Join(s.bootdir, "/grub/ubuntu-kernel_42.snap/initrd.img")
+	c.Assert(osutil.FileExists(initrdimg), Equals, true)
+
+	// it's idempotent
+	err = boot.ExtractKernelAssets(info, snapf)
+	c.Assert(err, IsNil)
+
+	// ensure that removal of assets also works
+	err = boot.RemoveKernelAssets(info)
+	c.Assert(err, IsNil)
+	exists, _, err := osutil.DirExists(filepath.Dir(kernimg))
+	c.Assert(err, IsNil)
+	c.Check(exists, Equals, false)
+
+	// it's idempotent
+	err = boot.RemoveKernelAssets(info)
+	c.Assert(err, IsNil)
+}
