@@ -26,6 +26,7 @@ import (
 	"github.com/snapcore/snapd/bootloader/androidbootenv"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snap"
 )
 
 type androidboot struct{}
@@ -43,12 +44,12 @@ func (a *androidboot) Name() string {
 	return "androidboot"
 }
 
-func (a *androidboot) Dir() string {
+func (a *androidboot) dir() string {
 	return filepath.Join(dirs.GlobalRootDir, "/boot/androidboot")
 }
 
 func (a *androidboot) ConfigFile() string {
-	return filepath.Join(a.Dir(), "androidboot.env")
+	return filepath.Join(a.dir(), "androidboot.env")
 }
 
 func (a *androidboot) GetBootVars(names ...string) (map[string]string, error) {
@@ -75,3 +76,12 @@ func (a *androidboot) SetBootVars(values map[string]string) error {
 	}
 	return env.Save()
 }
+
+func (a *androidboot) ExtractKernelAssets(s *snap.Info, snapf snap.Container) error {
+	return nil
+
+}
+
+func (a *androidboot) RemoveKernelAssets(s snap.PlaceInfo) error {
+	return nil
+}
