@@ -24,6 +24,8 @@ import (
 	"os"
 
 	. "gopkg.in/check.v1"
+
+	"github.com/snapcore/snapd/bootloader/ubootenv"
 )
 
 // creates a new Androidboot bootloader object
@@ -33,8 +35,36 @@ func NewAndroidBoot() Bootloader {
 
 func MockAndroidBootFile(c *C, mode os.FileMode) {
 	f := &androidboot{}
-	err := os.MkdirAll(f.Dir(), 0755)
+	err := os.MkdirAll(f.dir(), 0755)
 	c.Assert(err, IsNil)
 	err = ioutil.WriteFile(f.ConfigFile(), nil, mode)
 	c.Assert(err, IsNil)
 }
+
+func NewUboot() Bootloader {
+	return newUboot()
+}
+
+func MockUbootFiles(c *C) {
+	u := &uboot{}
+	err := os.MkdirAll(u.dir(), 0755)
+	c.Assert(err, IsNil)
+
+	// ensure that we have a valid uboot.env too
+	env, err := ubootenv.Create(u.envFile(), 4096)
+	c.Assert(err, IsNil)
+	err = env.Save()
+	c.Assert(err, IsNil)
+}
+
+func NewGrub() Bootloader {
+	return newGrub()
+}
+
+func MockGrubFiles(c *C) {
+	g := &grub{}
+	err := os.MkdirAll(g.dir(), 0755)
+	c.Assert(err, IsNil)
+	err = ioutil.WriteFile(g.ConfigFile(), nil, 0644)
+	c.Assert(err, IsNil)
+}
