@@ -25,6 +25,7 @@ import (
 	"github.com/snapcore/snapd/bootloader/ubootenv"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/snap"
 )
 
 type uboot struct{}
@@ -43,7 +44,7 @@ func (u *uboot) Name() string {
 	return "uboot"
 }
 
-func (u *uboot) Dir() string {
+func (u *uboot) dir() string {
 	return filepath.Join(dirs.GlobalRootDir, "/boot/uboot")
 }
 
@@ -52,7 +53,7 @@ func (u *uboot) ConfigFile() string {
 }
 
 func (u *uboot) envFile() string {
-	return filepath.Join(u.Dir(), "uboot.env")
+	return filepath.Join(u.dir(), "uboot.env")
 }
 
 func (u *uboot) SetBootVars(values map[string]string) error {
@@ -92,3 +93,11 @@ func (u *uboot) GetBootVars(names ...string) (map[string]string, error) {
 
 	return out, nil
 }
+
+func (u *uboot) ExtractKernelAssets(s *snap.Info, snapf snap.Container) error {
+	return extractKernelAssetsToBootDir(u.dir(), s, snapf)
+}
+
+func (u *uboot) RemoveKernelAssets(s snap.PlaceInfo) error {
+	return removeKernelAssetsFromBootDir(u.dir(), s)
+}
