@@ -77,21 +77,23 @@ func (cs *clientSuite) TestClientMultiOpSnapServerError(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpSnapResponseError(c *check.C) {
-	cs.rsp = `{"type": "error", "status": "potatoes"}`
+	cs.status = 400
+	cs.rsp = `{"type": "error"}`
 	for _, s := range ops {
 		_, err := s.op(cs.cli, pkgName, nil)
-		c.Check(err, check.ErrorMatches, `.*server error: "potatoes"`, check.Commentf(s.action))
+		c.Check(err, check.ErrorMatches, `.*server error: "Bad Request"`, check.Commentf(s.action))
 	}
 }
 
 func (cs *clientSuite) TestClientMultiOpSnapResponseError(c *check.C) {
-	cs.rsp = `{"type": "error", "status": "potatoes"}`
+	cs.status = 500
+	cs.rsp = `{"type": "error"}`
 	for _, s := range multiOps {
 		_, err := s.op(cs.cli, nil, nil)
-		c.Check(err, check.ErrorMatches, `.*server error: "potatoes"`, check.Commentf(s.action))
+		c.Check(err, check.ErrorMatches, `.*server error: "Internal Server Error"`, check.Commentf(s.action))
 	}
 	_, _, err := cs.cli.SnapshotMany(nil, nil)
-	c.Check(err, check.ErrorMatches, `.*server error: "potatoes"`)
+	c.Check(err, check.ErrorMatches, `.*server error: "Internal Server Error"`)
 }
 
 func (cs *clientSuite) TestClientOpSnapBadType(c *check.C) {
@@ -114,6 +116,7 @@ func (cs *clientSuite) TestClientOpSnapNotAccepted(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpSnapNoChange(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"status-code": 202,
 		"type": "async"
@@ -125,6 +128,7 @@ func (cs *clientSuite) TestClientOpSnapNoChange(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpSnap(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "d728",
 		"status-code": 202,
@@ -151,6 +155,7 @@ func (cs *clientSuite) TestClientOpSnap(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientMultiOpSnap(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "d728",
 		"status-code": 202,
@@ -179,6 +184,7 @@ func (cs *clientSuite) TestClientMultiOpSnap(c *check.C) {
 
 func (cs *clientSuite) TestClientMultiSnapshot(c *check.C) {
 	// Note body is essentially the same as TestClientMultiOpSnap; keep in sync
+	cs.status = 202
 	cs.rsp = `{
                 "result": {"set-id": 42},
 		"change": "d728",
@@ -203,6 +209,7 @@ func (cs *clientSuite) TestClientMultiSnapshot(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpInstallPath(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "66b3",
 		"status-code": 202,
@@ -230,6 +237,7 @@ func (cs *clientSuite) TestClientOpInstallPath(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpInstallPathInstance(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "66b3",
 		"status-code": 202,
@@ -258,6 +266,7 @@ func (cs *clientSuite) TestClientOpInstallPathInstance(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientOpInstallDangerous(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "66b3",
 		"status-code": 202,
@@ -293,6 +302,42 @@ func (cs *clientSuite) TestClientOpInstallDangerous(c *check.C) {
 	c.Assert(err, check.NotNil)
 }
 
+func (cs *clientSuite) TestClientOpInstallUnaliased(c *check.C) {
+	cs.status = 202
+	cs.rsp = `{
+		"change": "66b3",
+		"status-code": 202,
+		"type": "async"
+	}`
+	bodyData := []byte("snap-data")
+
+	snap := filepath.Join(c.MkDir(), "foo.snap")
+	err := ioutil.WriteFile(snap, bodyData, 0644)
+	c.Assert(err, check.IsNil)
+
+	opts := client.SnapOptions{
+		Unaliased: true,
+	}
+
+	_, err = cs.cli.Install("foo", &opts)
+	c.Assert(err, check.IsNil)
+
+	body, err := ioutil.ReadAll(cs.req.Body)
+	c.Assert(err, check.IsNil)
+	jsonBody := make(map[string]interface{})
+	err = json.Unmarshal(body, &jsonBody)
+	c.Assert(err, check.IsNil, check.Commentf("body: %v", string(body)))
+	c.Check(jsonBody["unaliased"], check.Equals, true, check.Commentf("body: %v", string(body)))
+
+	_, err = cs.cli.InstallPath(snap, "", &opts)
+	c.Assert(err, check.IsNil)
+
+	body, err = ioutil.ReadAll(cs.req.Body)
+	c.Assert(err, check.IsNil)
+
+	c.Assert(string(body), check.Matches, "(?s).*Content-Disposition: form-data; name=\"unaliased\"\r\n\r\ntrue\r\n.*")
+}
+
 func formToMap(c *check.C, mr *multipart.Reader) map[string]string {
 	formData := map[string]string{}
 	for {
@@ -309,6 +354,7 @@ func formToMap(c *check.C, mr *multipart.Reader) map[string]string {
 }
 
 func (cs *clientSuite) TestClientOpTryMode(c *check.C) {
+	cs.status = 202
 	cs.rsp = `{
 		"change": "66b3",
 		"status-code": 202,
@@ -365,3 +411,26 @@ func (cs *clientSuite) TestClientOpTryModeDangerous(c *check.C) {
 	_, err := cs.cli.Try(snapdir, &client.SnapOptions{Dangerous: true})
 	c.Assert(err, check.Equals, client.ErrDangerousNotApplicable)
 }
+
+func (cs *clientSuite) TestSnapOptionsSerialises(c *check.C) {
+	tests := map[string]client.SnapOptions{
+		"{}":                         {},
+		`{"channel":"edge"}`:         {Channel: "edge"},
+		`{"revision":"42"}`:          {Revision: "42"},
+		`{"cohort-key":"what"}`:      {CohortKey: "what"},
+		`{"leave-cohort":true}`:      {LeaveCohort: true},
+		`{"devmode":true}`:           {DevMode: true},
+		`{"jailmode":true}`:          {JailMode: true},
+		`{"classic":true}`:           {Classic: true},
+		`{"dangerous":true}`:         {Dangerous: true},
+		`{"ignore-validation":true}`: {IgnoreValidation: true},
+		`{"unaliased":true}`:         {Unaliased: true},
+		`{"purge":true}`:             {Purge: true},
+		`{"amend":true}`:             {Amend: true},
+	}
+	for expected, opts := range tests {
+		buf, err := json.Marshal(&opts)
+		c.Assert(err, check.IsNil, check.Commentf("%s", expected))
+		c.Check(string(buf), check.Equals, expected)
+	}
+}
