@@ -21,7 +21,7 @@
 #include <stdbool.h>
 
 typedef enum sc_feature_flag {
-	SC_PER_USER_MOUNT_NAMESPACE,
+	SC_FEATURE_PER_USER_MOUNT_NAMESPACE,
 	SC_FEATURE_REFRESH_APP_AWARENESS,
 } sc_feature_flag;
 
