@@ -23,9 +23,11 @@
 #include <glob.h>
 #include <sched.h>
 #include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/capability.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -227,7 +229,8 @@ static void sc_restore_process_state(const sc_preserved_process_state *
 	 * designated user so UNIX permissions are in effect. */
 	if (fchdir(inner_cwd_fd) < 0) {
 		if (errno == EPERM || errno == EACCES) {
-			debug("cannot access original working directory %s", orig_cwd);
+			debug("cannot access original working directory %s",
+			      orig_cwd);
 			goto the_void;
 		}
 		die("cannot restore original working directory via path");
@@ -237,9 +240,9 @@ static void sc_restore_process_state(const sc_preserved_process_state *
 	 * instruct the user and avoid potential confusion. This mostly applies to
 	 * tools that are invoked from /tmp. */
 	if (proc_state->file_info_orig_cwd.st_dev ==
-		file_info_inner.st_dev
-		&& proc_state->file_info_orig_cwd.st_ino ==
-		file_info_inner.st_ino) {
+	    file_info_inner.st_dev
+	    && proc_state->file_info_orig_cwd.st_ino ==
+	    file_info_inner.st_ino) {
 		/* The path of the original working directory points to the same
 		 * inode as before. */
 		debug("working directory restored to %s", orig_cwd);
@@ -301,7 +304,8 @@ int main(int argc, char **argv)
 	struct sc_args *args SC_CLEANUP(sc_cleanup_args) = NULL;
 	sc_preserved_process_state proc_state
 	    SC_CLEANUP(sc_cleanup_preserved_process_state) = {
-	.orig_umask = 0,.orig_cwd_fd = -1};
+		.orig_umask = 0,.orig_cwd_fd = -1
+	};
 	args = sc_nonfatal_parse_args(&argc, &argv, &err);
 	sc_die_on_error(err);
 
@@ -329,8 +333,12 @@ int main(int argc, char **argv)
 	// Who are we?
 	uid_t real_uid, effective_uid, saved_uid;
 	gid_t real_gid, effective_gid, saved_gid;
-	getresuid(&real_uid, &effective_uid, &saved_uid);
-	getresgid(&real_gid, &effective_gid, &saved_gid);
+	if (getresuid(&real_uid, &effective_uid, &saved_uid) != 0) {
+		die("getresuid failed");
+	}
+	if (getresgid(&real_gid, &effective_gid, &saved_gid) != 0) {
+		die("getresgid failed");
+	}
 	debug("ruid: %d, euid: %d, suid: %d",
 	      real_uid, effective_uid, saved_uid);
 	debug("rgid: %d, egid: %d, sgid: %d",
@@ -358,9 +366,13 @@ int main(int argc, char **argv)
 		sc_error *err SC_CLEANUP(sc_cleanup_error) = NULL;
 		snap_context =
 		    sc_cookie_get_from_snapd(invocation.snap_instance, &err);
-		if (err != NULL) {
-			error("%s\n", sc_error_msg(err));
-		}
+		/* While the cookie is normally present due to various protection
+		 * mechanisms ensuring its creation from snapd, we are not considering
+		 * it a critical error for snap-confine in the case it is absent. When
+		 * absent snaps attempting to utilize snapctl to interact with snapd
+		 * will fail but it is more important to run a little than break
+		 * entirely in case snapd-side code is incorrect. Therefore error
+		 * information is collected but discarded. */
 	}
 
 	struct sc_apparmor apparmor;
@@ -410,18 +422,48 @@ int main(int argc, char **argv)
 	// For classic and confined snaps
 	sc_selinux_set_snap_execcon();
 #endif
-	if (sc_apply_seccomp_profile_for_security_tag(invocation.security_tag)) {
-		/* If the process is not explicitly unconfined then load the global
-		 * profile as well. */
-		sc_apply_global_seccomp_profile();
-	}
 	if (snap_context != NULL) {
 		setenv("SNAP_COOKIE", snap_context, 1);
 		// for compatibility, if facing older snapd.
 		setenv("SNAP_CONTEXT", snap_context, 1);
 	}
+	// Normally setuid/setgid not only permanently drops the UID/GID, but
+	// also clears the capabilities bounding sets (see "Effect of user ID
+	// changes on capabilities" in 'man capabilities'). To load a seccomp
+	// profile, we need either CAP_SYS_ADMIN or PR_SET_NO_NEW_PRIVS. Since
+	// NNP causes issues with AppArmor and exec transitions in certain
+	// snapd interfaces, keep CAP_SYS_ADMIN temporarily when we are
+	// permanently dropping privileges.
+	if (getresuid(&real_uid, &effective_uid, &saved_uid) != 0) {
+		die("getresuid failed");
+	}
+	debug("ruid: %d, euid: %d, suid: %d",
+	      real_uid, effective_uid, saved_uid);
+	struct __user_cap_header_struct hdr =
+	    { _LINUX_CAPABILITY_VERSION_3, 0 };
+	struct __user_cap_data_struct cap_data[2] = { {0} };
+
+	// At this point in time, if we are going to permanently drop our
+	// effective_uid will not be '0' but our saved_uid will be '0'. Detect
+	// and save when we are in the this state so know when to setup the
+	// capabilities bounding set, regain CAP_SYS_ADMIN and later drop it.
+	bool keep_sys_admin = effective_uid != 0 && saved_uid == 0;
+	if (keep_sys_admin) {
+		debug("setting capabilities bounding set");
+		// clear all 32 bit caps but SYS_ADMIN, with none inheritable
+		cap_data[0].effective = CAP_TO_MASK(CAP_SYS_ADMIN);
+		cap_data[0].permitted = cap_data[0].effective;
+		cap_data[0].inheritable = 0;
+		// clear all 64 bit caps
+		cap_data[1].effective = 0;
+		cap_data[1].permitted = 0;
+		cap_data[1].inheritable = 0;
+		if (capset(&hdr, cap_data) != 0) {
+			die("capset failed");
+		}
+	}
 	// Permanently drop if not root
-	if (geteuid() == 0) {
+	if (effective_uid == 0) {
 		// Note that we do not call setgroups() here because its ok
 		// that the user keeps the groups he already belongs to
 		if (setgid(real_gid) != 0)
@@ -434,6 +476,32 @@ int main(int argc, char **argv)
 		if (real_uid != 0 && (getgid() == 0 || getegid() == 0))
 			die("permanently dropping privs did not work");
 	}
+	// Now that we've permanently dropped, regain SYS_ADMIN
+	if (keep_sys_admin) {
+		debug("regaining SYS_ADMIN");
+		cap_data[0].effective = CAP_TO_MASK(CAP_SYS_ADMIN);
+		cap_data[0].permitted = cap_data[0].effective;
+		if (capset(&hdr, cap_data) != 0) {
+			die("capset regain failed");
+		}
+	}
+	// Now that we've dropped and regained SYS_ADMIN, we can load the
+	// seccomp profiles.
+	if (sc_apply_seccomp_profile_for_security_tag(invocation.security_tag)) {
+		// If the process is not explicitly unconfined then load the
+		// global profile as well.
+		sc_apply_global_seccomp_profile();
+	}
+	// Even though we set inheritable to 0, let's clear SYS_ADMIN
+	// explicitly
+	if (keep_sys_admin) {
+		debug("clearing SYS_ADMIN");
+		cap_data[0].effective = 0;
+		cap_data[0].permitted = cap_data[0].effective;
+		if (capset(&hdr, cap_data) != 0) {
+			die("capset clear failed");
+		}
+	}
 	// and exec the new executable
 	argv[0] = (char *)invocation.executable;
 	debug("execv(%s, %s...)", invocation.executable, argv[0]);
@@ -504,6 +572,12 @@ static void enter_non_classic_execution_environment(sc_invocation * inv,
 	// Init and check rootfs_dir, apply any fallback behaviors.
 	sc_check_rootfs_dir(inv);
 
+	/** Populate and join the device control group. */
+	struct snappy_udev udev_s;
+	if (snappy_udev_init(inv->security_tag, &udev_s) == 0)
+		setup_devices_cgroup(inv->security_tag, &udev_s);
+	snappy_udev_cleanup(&udev_s);
+
 	/**
 	 * is_normal_mode controls if we should pivot into the base snap.
 	 *
@@ -532,7 +606,7 @@ static void enter_non_classic_execution_environment(sc_invocation * inv,
 	 **/
 	sc_distro distro = sc_classify_distro();
 	inv->is_normal_mode = distro != SC_DISTRO_CORE16 ||
-		!sc_streq(inv->orig_base_snap_name, "core");
+	    !sc_streq(inv->orig_base_snap_name, "core");
 
 	/* Stale mount namespace discarded or no mount namespace to
 	   join. We need to construct a new mount namespace ourselves.
@@ -548,6 +622,7 @@ static void enter_non_classic_execution_environment(sc_invocation * inv,
 			die("cannot unshare the mount namespace");
 		}
 		sc_populate_mount_ns(aa, snap_update_ns_fd, inv);
+		sc_store_ns_info(inv);
 
 		/* Preserve the mount namespace. */
 		sc_preserve_populated_mount_ns(group);
@@ -637,8 +712,4 @@ static void enter_non_classic_execution_environment(sc_invocation * inv,
 			die("cannot set environment variable '%s'", tmpd[i]);
 		}
 	}
-	struct snappy_udev udev_s;
-	if (snappy_udev_init(inv->security_tag, &udev_s) == 0)
-		setup_devices_cgroup(inv->security_tag, &udev_s);
-	snappy_udev_cleanup(&udev_s);
 }
