@@ -165,6 +165,9 @@ var SeccompSyscalls = []string{
 	"io_pgetevents",
 	"io_setup",
 	"io_submit",
+	"io_uring_enter",
+	"io_uring_register",
+	"io_uring_setup",
 	"ioctl",
 	"ioperm",
 	"iopl",
