@@ -311,8 +311,8 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 				// now change mount propagation (shared/rshared, private/rprivate,
 				// slave/rslave, unbindable/runbindable).
 				flagsForMount := uintptr(maskedFlagsPropagation | maskedFlagsRecursive)
-				err = sysMount("", c.Entry.Dir, "", flagsForMount, "")
-				logger.Debugf("mount %q %q %q %d %q (error: %v)", "", c.Entry.Dir, "", flagsForMount, "", err)
+				err = sysMount("none", c.Entry.Dir, "", flagsForMount, "")
+				logger.Debugf("mount %q %q %q %d %q (error: %v)", "none", c.Entry.Dir, "", flagsForMount, "", err)
 			}
 			if err == nil {
 				as.AddChange(c)
@@ -392,6 +392,7 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 		}
 		return err
 	case Keep:
+		as.AddChange(c)
 		return nil
 	}
 	return fmt.Errorf("cannot process mount change: unknown action: %q", c.Action)
@@ -403,7 +404,10 @@ func (c *Change) lowLevelPerform(as *Assumptions) error {
 // lists are processed and a "diff" of mount changes is produced. The mount
 // changes, when applied in order, transform the current profile into the
 // desired profile.
-func NeededChanges(currentProfile, desiredProfile *osutil.MountProfile) []*Change {
+var NeededChanges = neededChangesImpl
+
+// neededChangesImpl is the real implementation of NeededChanges
+func neededChangesImpl(currentProfile, desiredProfile *osutil.MountProfile) []*Change {
 	// Copy both profiles as we will want to mutate them.
 	current := make([]osutil.MountEntry, len(currentProfile.Entries))
 	copy(current, currentProfile.Entries)
@@ -495,7 +499,15 @@ func NeededChanges(currentProfile, desiredProfile *osutil.MountProfile) []*Chang
 		if reuse[current[i].Dir] {
 			changes = append(changes, &Change{Action: Keep, Entry: current[i]})
 		} else {
-			changes = append(changes, &Change{Action: Unmount, Entry: current[i]})
+			var entry osutil.MountEntry = current[i]
+			entry.Options = append([]string(nil), entry.Options...)
+			// If the mount entry can potentially host nested mount points then detach
+			// rather than unmount, since detach will always succeed.
+			shouldDetach := entry.Type == "tmpfs" || entry.OptBool("bind") || entry.OptBool("rbind")
+			if shouldDetach && !entry.XSnapdDetach() {
+				entry.Options = append(entry.Options, osutil.XSnapdDetach())
+			}
+			changes = append(changes, &Change{Action: Unmount, Entry: entry})
 		}
 	}
 
