@@ -48,8 +48,18 @@ func addHelp(parser *flags.Parser) error {
 		// on which help is being requested (like "snap foo
 		// --help", active is foo), or nil in the toplevel.
 		if parser.Command.Active == nil {
-			// toplevel --help will get handled via ErrCommandRequired
-			return nil
+			// this means *either* a bare 'snap --help',
+			// *or* 'snap --help command'
+			//
+			// If we return nil in the first case go-flags
+			// will throw up an ErrCommandRequired on its
+			// own, but in the second case it'll go on to
+			// run the command, which is very unexpected.
+			//
+			// So we force the ErrCommandRequired here.
+
+			// toplevel --help gets handled via ErrCommandRequired
+			return &flags.Error{Type: flags.ErrCommandRequired}
 		}
 		// not toplevel, so ask for regular help
 		return &flags.Error{Type: flags.ErrHelp}
@@ -190,7 +200,7 @@ var helpCategories = []helpCategory{
 	}, {
 		Label:       i18n.G("Configuration"),
 		Description: i18n.G("system administration and configuration"),
-		Commands:    []string{"get", "set", "wait"},
+		Commands:    []string{"get", "set", "unset", "wait"},
 	}, {
 		Label:       i18n.G("Account"),
 		Description: i18n.G("authentication to snapd and the snap store"),
@@ -206,7 +216,7 @@ var helpCategories = []helpCategory{
 	}, {
 		Label:       i18n.G("Other"),
 		Description: i18n.G("miscellanea"),
-		Commands:    []string{"version", "warnings", "okay", "ack", "known"},
+		Commands:    []string{"version", "warnings", "okay", "ack", "known", "create-cohort"},
 	}, {
 		Label:       i18n.G("Development"),
 		Description: i18n.G("developer-oriented features"),
