@@ -32,9 +32,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
-	"os/user"
 	"path/filepath"
-	"regexp"
 	"sort"
 	"strconv"
 	"strings"
@@ -58,7 +56,6 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate"
 	"github.com/snapcore/snapd/overlord/configstate/config"
-	"github.com/snapcore/snapd/overlord/devicestate"
 	"github.com/snapcore/snapd/overlord/hookstate/ctlcmd"
 	"github.com/snapcore/snapd/overlord/ifacestate"
 	"github.com/snapcore/snapd/overlord/servicestate"
@@ -83,6 +80,7 @@ var api = []*Command{
 	snapsCmd,
 	snapCmd,
 	snapFileCmd,
+	snapDownloadCmd,
 	snapConfCmd,
 	interfacesCmd,
 	assertsCmd,
@@ -99,10 +97,12 @@ var api = []*Command{
 	appsCmd,
 	logsCmd,
 	warningsCmd,
+	debugPprofCmd,
 	debugCmd,
 	snapshotCmd,
 	connectionsCmd,
 	modelCmd,
+	cohortsCmd,
 }
 
 var (
@@ -119,17 +119,6 @@ var (
 		GET:     sysInfo,
 	}
 
-	loginCmd = &Command{
-		Path:     "/v2/login",
-		POST:     loginUser,
-		PolkitOK: "io.snapcraft.snapd.login",
-	}
-
-	logoutCmd = &Command{
-		Path: "/v2/logout",
-		POST: logoutUser,
-	}
-
 	appIconCmd = &Command{
 		Path:   "/v2/icons/{name}/icon",
 		UserOK: true,
@@ -199,11 +188,6 @@ var (
 		GET:    getChanges,
 	}
 
-	createUserCmd = &Command{
-		Path: "/v2/create-user",
-		POST: postCreateUser,
-	}
-
 	buyCmd = &Command{
 		Path: "/v2/buy",
 		POST: postBuy,
@@ -220,11 +204,6 @@ var (
 		POST:   runSnapctl,
 	}
 
-	usersCmd = &Command{
-		Path: "/v2/users",
-		GET:  getUsers,
-	}
-
 	sectionsCmd = &Command{
 		Path:   "/v2/sections",
 		UserOK: true,
@@ -354,147 +333,6 @@ func sandboxFeatures(backends []interfaces.SecurityBackend) map[string][]string
 	return result
 }
 
-// userResponseData contains the data releated to user creation/login/query
-type userResponseData struct {
-	ID       int      `json:"id,omitempty"`
-	Username string   `json:"username,omitempty"`
-	Email    string   `json:"email,omitempty"`
-	SSHKeys  []string `json:"ssh-keys,omitempty"`
-
-	Macaroon   string   `json:"macaroon,omitempty"`
-	Discharges []string `json:"discharges,omitempty"`
-}
-
-var isEmailish = regexp.MustCompile(`.@.*\..`).MatchString
-
-func loginUser(c *Command, r *http.Request, user *auth.UserState) Response {
-	var loginData struct {
-		Username string `json:"username"`
-		Email    string `json:"email"`
-		Password string `json:"password"`
-		Otp      string `json:"otp"`
-	}
-
-	decoder := json.NewDecoder(r.Body)
-	if err := decoder.Decode(&loginData); err != nil {
-		return BadRequest("cannot decode login data from request body: %v", err)
-	}
-
-	if loginData.Email == "" && isEmailish(loginData.Username) {
-		// for backwards compatibility, if no email is provided assume username is the email
-		loginData.Email = loginData.Username
-		loginData.Username = ""
-	}
-
-	if loginData.Email == "" && user != nil && user.Email != "" {
-		loginData.Email = user.Email
-	}
-
-	// the "username" needs to look a lot like an email address
-	if !isEmailish(loginData.Email) {
-		return SyncResponse(&resp{
-			Type: ResponseTypeError,
-			Result: &errorResult{
-				Message: "please use a valid email address.",
-				Kind:    errorKindInvalidAuthData,
-				Value:   map[string][]string{"email": {"invalid"}},
-			},
-			Status: 400,
-		}, nil)
-	}
-
-	overlord := c.d.overlord
-	st := overlord.State()
-	theStore := getStore(c)
-	macaroon, discharge, err := theStore.LoginUser(loginData.Email, loginData.Password, loginData.Otp)
-	switch err {
-	case store.ErrAuthenticationNeeds2fa:
-		return SyncResponse(&resp{
-			Type: ResponseTypeError,
-			Result: &errorResult{
-				Kind:    errorKindTwoFactorRequired,
-				Message: err.Error(),
-			},
-			Status: 401,
-		}, nil)
-	case store.Err2faFailed:
-		return SyncResponse(&resp{
-			Type: ResponseTypeError,
-			Result: &errorResult{
-				Kind:    errorKindTwoFactorFailed,
-				Message: err.Error(),
-			},
-			Status: 401,
-		}, nil)
-	default:
-		switch err := err.(type) {
-		case store.InvalidAuthDataError:
-			return SyncResponse(&resp{
-				Type: ResponseTypeError,
-				Result: &errorResult{
-					Message: err.Error(),
-					Kind:    errorKindInvalidAuthData,
-					Value:   err,
-				},
-				Status: 400,
-			}, nil)
-		case store.PasswordPolicyError:
-			return SyncResponse(&resp{
-				Type: ResponseTypeError,
-				Result: &errorResult{
-					Message: err.Error(),
-					Kind:    errorKindPasswordPolicy,
-					Value:   err,
-				},
-				Status: 401,
-			}, nil)
-		}
-		return Unauthorized(err.Error())
-	case nil:
-		// continue
-	}
-	st.Lock()
-	if user != nil {
-		// local user logged-in, set its store macaroons
-		user.StoreMacaroon = macaroon
-		user.StoreDischarges = []string{discharge}
-		// user's email address authenticated by the store
-		user.Email = loginData.Email
-		err = auth.UpdateUser(st, user)
-	} else {
-		user, err = auth.NewUser(st, loginData.Username, loginData.Email, macaroon, []string{discharge})
-	}
-	st.Unlock()
-	if err != nil {
-		return InternalError("cannot persist authentication details: %v", err)
-	}
-
-	result := userResponseData{
-		ID:         user.ID,
-		Username:   user.Username,
-		Email:      user.Email,
-		Macaroon:   user.Macaroon,
-		Discharges: user.Discharges,
-	}
-	return SyncResponse(result, nil)
-}
-
-func logoutUser(c *Command, r *http.Request, user *auth.UserState) Response {
-	state := c.d.overlord.State()
-	state.Lock()
-	defer state.Unlock()
-
-	if user == nil {
-		return BadRequest("not logged in")
-	}
-	err := auth.RemoveUser(state, user.ID)
-	if err != nil {
-		return InternalError(err.Error())
-	}
-
-	return SyncResponse(nil, nil)
-}
-
 // UserFromRequest extracts user information from request and return the respective user in state, if valid
 // It requires the state to be locked
 func UserFromRequest(st *state.State, req *http.Request) (*auth.UserState, error) {
@@ -580,7 +418,7 @@ func getStore(c *Command) snapstate.StoreService {
 	st.Lock()
 	defer st.Unlock()
 
-	return snapstate.Store(st)
+	return snapstate.Store(st, nil)
 }
 
 func getSections(c *Command, r *http.Request, user *auth.UserState) Response {
@@ -664,7 +502,8 @@ func searchStore(c *Command, r *http.Request, user *auth.UserState) Response {
 	}
 
 	theStore := getStore(c)
-	found, err := theStore.Find(&store.Search{
+	ctx := store.WithClientUserAgent(r.Context(), r)
+	found, err := theStore.Find(ctx, &store.Search{
 		Query:    q,
 		Prefix:   prefix,
 		CommonID: commonID,
@@ -730,7 +569,8 @@ func findOne(c *Command, r *http.Request, user *auth.UserState, name string) Res
 	spec := store.SnapSpec{
 		Name: name,
 	}
-	snapInfo, err := theStore.SnapInfo(spec, user)
+	ctx := store.WithClientUserAgent(r.Context(), r)
+	snapInfo, err := theStore.SnapInfo(ctx, spec, user)
 	switch err {
 	case nil:
 		// pass
@@ -896,11 +736,14 @@ type snapInstruction struct {
 	Amend            bool          `json:"amend"`
 	Channel          string        `json:"channel"`
 	Revision         snap.Revision `json:"revision"`
+	CohortKey        string        `json:"cohort-key"`
+	LeaveCohort      bool          `json:"leave-cohort"`
 	DevMode          bool          `json:"devmode"`
 	JailMode         bool          `json:"jailmode"`
 	Classic          bool          `json:"classic"`
 	IgnoreValidation bool          `json:"ignore-validation"`
 	Unaliased        bool          `json:"unaliased"`
+	Purge            bool          `json:"purge,omitempty"`
 	// dropping support temporarely until flag confusion is sorted,
 	// this isn't supported by client atm anyway
 	LeaveOld bool         `json:"temp-dropped-leave-old"`
@@ -910,6 +753,16 @@ type snapInstruction struct {
 
 	// The fields below should not be unmarshalled into. Do not export them.
 	userID int
+	ctx    context.Context
+}
+
+func (inst *snapInstruction) revnoOpts() *snapstate.RevisionOptions {
+	return &snapstate.RevisionOptions{
+		Channel:     inst.Channel,
+		Revision:    inst.Revision,
+		CohortKey:   inst.CohortKey,
+		LeaveCohort: inst.LeaveCohort,
+	}
 }
 
 func (inst *snapInstruction) modeFlags() (snapstate.Flags, error) {
@@ -945,6 +798,7 @@ var (
 	snapstateRemoveMany        = snapstate.RemoveMany
 	snapstateRevert            = snapstate.Revert
 	snapstateRevertToRevision  = snapstate.RevertToRevision
+	snapstateSwitch            = snapstate.Switch
 
 	snapshotList    = snapshotstate.List
 	snapshotCheck   = snapshotstate.Check
@@ -1022,6 +876,22 @@ func snapUpdateMany(inst *snapInstruction, st *state.State) (*snapInstructionRes
 }
 
 func verifySnapInstructions(inst *snapInstruction) error {
+	if inst.CohortKey != "" {
+		if inst.LeaveCohort {
+			return fmt.Errorf("cannot specify both cohort-key and leave-cohort")
+		}
+		if !inst.Revision.Unset() {
+			return fmt.Errorf("cannot specify both cohort-key and revision")
+		}
+		if inst.Action != "install" && inst.Action != "refresh" && inst.Action != "switch" {
+			return fmt.Errorf("cohort-key can only be specified for install, refresh, or switch")
+		}
+	}
+	if inst.LeaveCohort {
+		if inst.Action != "refresh" && inst.Action != "switch" {
+			return fmt.Errorf("leave-cohort can only be specified for refresh or switch")
+		}
+	}
 	switch inst.Action {
 	case "install":
 		for _, snapName := range inst.Snaps {
@@ -1076,16 +946,24 @@ func snapInstall(inst *snapInstruction, st *state.State) (string, []*state.TaskS
 		return "", nil, err
 	}
 
-	logger.Noticef("Installing snap %q revision %s", inst.Snaps[0], inst.Revision)
-
-	tset, err := snapstateInstall(st, inst.Snaps[0], inst.Channel, inst.Revision, inst.userID, flags)
+	var ckey string
+	if inst.CohortKey == "" {
+		logger.Noticef("Installing snap %q revision %s", inst.Snaps[0], inst.Revision)
+	} else {
+		ckey = strutil.ElliptLeft(inst.CohortKey, 10)
+		logger.Noticef("Installing snap %q from cohort %q", inst.Snaps[0], ckey)
+	}
+	tset, err := snapstateInstall(inst.ctx, st, inst.Snaps[0], inst.revnoOpts(), inst.userID, flags)
 	if err != nil {
 		return "", nil, err
 	}
 
 	msg := fmt.Sprintf(i18n.G("Install %q snap"), inst.Snaps[0])
 	if inst.Channel != "stable" && inst.Channel != "" {
-		msg = fmt.Sprintf(i18n.G("Install %q snap from %q channel"), inst.Snaps[0], inst.Channel)
+		msg += fmt.Sprintf(" from %q channel", inst.Channel)
+	}
+	if inst.CohortKey != "" {
+		msg += fmt.Sprintf(" from %q cohort", ckey)
 	}
 	return msg, []*state.TaskSet{tset}, nil
 }
@@ -1108,7 +986,7 @@ func snapUpdate(inst *snapInstruction, st *state.State) (string, []*state.TaskSe
 		return "", nil, err
 	}
 
-	ts, err := snapstateUpdate(st, inst.Snaps[0], inst.Channel, inst.Revision, inst.userID, flags)
+	ts, err := snapstateUpdate(st, inst.Snaps[0], inst.revnoOpts(), inst.userID, flags)
 	if err != nil {
 		return "", nil, err
 	}
@@ -1147,7 +1025,7 @@ func snapRemoveMany(inst *snapInstruction, st *state.State) (*snapInstructionRes
 }
 
 func snapRemove(inst *snapInstruction, st *state.State) (string, []*state.TaskSet, error) {
-	ts, err := snapstate.Remove(st, inst.Snaps[0], inst.Revision)
+	ts, err := snapstate.Remove(st, inst.Snaps[0], inst.Revision, &snapstate.RemoveFlags{Purge: inst.Purge})
 	if err != nil {
 		return "", nil, err
 	}
@@ -1207,12 +1085,24 @@ func snapSwitch(inst *snapInstruction, st *state.State) (string, []*state.TaskSe
 	if !inst.Revision.Unset() {
 		return "", nil, errors.New("switch takes no revision")
 	}
-	ts, err := snapstate.Switch(st, inst.Snaps[0], inst.Channel)
+	ts, err := snapstateSwitch(st, inst.Snaps[0], inst.revnoOpts())
 	if err != nil {
 		return "", nil, err
 	}
 
-	msg := fmt.Sprintf(i18n.G("Switch %q snap to %s"), inst.Snaps[0], inst.Channel)
+	var msg string
+	switch {
+	case inst.LeaveCohort && inst.Channel != "":
+		msg = fmt.Sprintf(i18n.G("Switch %q snap to channel %q and away from cohort"), inst.Snaps[0], inst.Channel)
+	case inst.LeaveCohort:
+		msg = fmt.Sprintf(i18n.G("Switch %q snap away from cohort"), inst.Snaps[0])
+	case inst.CohortKey == "" && inst.Channel != "":
+		msg = fmt.Sprintf(i18n.G("Switch %q snap to channel %q"), inst.Snaps[0], inst.Channel)
+	case inst.CohortKey != "" && inst.Channel == "":
+		msg = fmt.Sprintf(i18n.G("Switch %q snap to cohort %q"), inst.Snaps[0], strutil.ElliptLeft(inst.CohortKey, 10))
+	default:
+		msg = fmt.Sprintf(i18n.G("Switch %q snap to channel %q and cohort %q"), inst.Snaps[0], inst.Channel, strutil.ElliptLeft(inst.CohortKey, 10))
+	}
 	return msg, []*state.TaskSet{ts}, nil
 }
 
@@ -1276,6 +1166,7 @@ func postSnap(c *Command, r *http.Request, user *auth.UserState) Response {
 	if err := decoder.Decode(&inst); err != nil {
 		return BadRequest("cannot decode request body into snap instruction: %v", err)
 	}
+	inst.ctx = r.Context()
 
 	state := c.d.overlord.State()
 	state.Lock()
@@ -1389,12 +1280,13 @@ func snapsOp(c *Command, r *http.Request, user *auth.UserState) Response {
 		return BadRequest("cannot decode request body into snap instruction: %v", err)
 	}
 
+	// TODO: inst.Amend, etc?
+	if inst.Channel != "" || !inst.Revision.Unset() || inst.DevMode || inst.JailMode || inst.CohortKey != "" || inst.LeaveCohort {
+		return BadRequest("unsupported option provided for multi-snap operation")
+	}
 	if err := verifySnapInstructions(&inst); err != nil {
 		return BadRequest("%v", err)
 	}
-	if inst.Channel != "" || !inst.Revision.Unset() || inst.DevMode || inst.JailMode {
-		return BadRequest("unsupported option provided for multi-snap operation")
-	}
 
 	st := c.d.overlord.State()
 	st.Lock()
@@ -1645,7 +1537,7 @@ func iconGet(st *state.State, name string) Response {
 		return NotFound("local snap has no icon")
 	}
 
-	return FileResponse(icon)
+	return fileResponse(icon)
 }
 
 func appIconGet(c *Command, r *http.Request, user *auth.UserState) Response {
@@ -2109,260 +2001,10 @@ func abortChange(c *Command, r *http.Request, user *auth.UserState) Response {
 }
 
 var (
-	postCreateUserUcrednetGet = ucrednetGet
-	runSnapctlUcrednetGet     = ucrednetGet
-	ctlcmdRun                 = ctlcmd.Run
-	osutilAddUser             = osutil.AddUser
+	runSnapctlUcrednetGet = ucrednetGet
+	ctlcmdRun             = ctlcmd.Run
 )
 
-func getUserDetailsFromStore(theStore snapstate.StoreService, email string) (string, *osutil.AddUserOptions, error) {
-	v, err := theStore.UserInfo(email)
-	if err != nil {
-		return "", nil, fmt.Errorf("cannot create user %q: %s", email, err)
-	}
-	if len(v.SSHKeys) == 0 {
-		return "", nil, fmt.Errorf("cannot create user for %q: no ssh keys found", email)
-	}
-
-	gecos := fmt.Sprintf("%s,%s", email, v.OpenIDIdentifier)
-	opts := &osutil.AddUserOptions{
-		SSHKeys: v.SSHKeys,
-		Gecos:   gecos,
-	}
-	return v.Username, opts, nil
-}
-
-func createAllKnownSystemUsers(st *state.State, createData *postUserCreateData) Response {
-	var createdUsers []userResponseData
-
-	st.Lock()
-	db := assertstate.DB(st)
-	modelAs, err := devicestate.Model(st)
-	st.Unlock()
-	if err != nil {
-		return InternalError("cannot get model assertion")
-	}
-
-	headers := map[string]string{
-		"brand-id": modelAs.BrandID(),
-	}
-	st.Lock()
-	assertions, err := db.FindMany(asserts.SystemUserType, headers)
-	st.Unlock()
-	if err != nil && !asserts.IsNotFound(err) {
-		return BadRequest("cannot find system-user assertion: %s", err)
-	}
-
-	for _, as := range assertions {
-		email := as.(*asserts.SystemUser).Email()
-		// we need to use getUserDetailsFromAssertion as this verifies
-		// the assertion against the current brand/model/time
-		username, opts, err := getUserDetailsFromAssertion(st, email)
-		if err != nil {
-			logger.Noticef("ignoring system-user assertion for %q: %s", email, err)
-			continue
-		}
-		// ignore already existing users
-		if _, err := user.Lookup(username); err == nil {
-			continue
-		}
-
-		// FIXME: duplicated code
-		opts.Sudoer = createData.Sudoer
-		opts.ExtraUsers = !release.OnClassic
-
-		if err := osutilAddUser(username, opts); err != nil {
-			return InternalError("cannot add user %q: %s", username, err)
-		}
-		if err := setupLocalUser(st, username, email); err != nil {
-			return InternalError("%s", err)
-		}
-		createdUsers = append(createdUsers, userResponseData{
-			Username: username,
-			SSHKeys:  opts.SSHKeys,
-		})
-	}
-
-	return SyncResponse(createdUsers, nil)
-}
-
-func getUserDetailsFromAssertion(st *state.State, email string) (string, *osutil.AddUserOptions, error) {
-	errorPrefix := fmt.Sprintf("cannot add system-user %q: ", email)
-
-	st.Lock()
-	db := assertstate.DB(st)
-	modelAs, err := devicestate.Model(st)
-	st.Unlock()
-	if err != nil {
-		return "", nil, fmt.Errorf(errorPrefix+"cannot get model assertion: %s", err)
-	}
-
-	brandID := modelAs.BrandID()
-	series := modelAs.Series()
-	model := modelAs.Model()
-
-	a, err := db.Find(asserts.SystemUserType, map[string]string{
-		"brand-id": brandID,
-		"email":    email,
-	})
-	if err != nil {
-		return "", nil, fmt.Errorf(errorPrefix+"%v", err)
-	}
-	// the asserts package guarantees that this cast will work
-	su := a.(*asserts.SystemUser)
-
-	// cross check that the assertion is valid for the given series/model
-
-	// check that the signer of the assertion is one of the accepted ones
-	sysUserAuths := modelAs.SystemUserAuthority()
-	if len(sysUserAuths) > 0 && !strutil.ListContains(sysUserAuths, su.AuthorityID()) {
-		return "", nil, fmt.Errorf(errorPrefix+"%q not in accepted authorities %q", email, su.AuthorityID(), sysUserAuths)
-	}
-	if len(su.Series()) > 0 && !strutil.ListContains(su.Series(), series) {
-		return "", nil, fmt.Errorf(errorPrefix+"%q not in series %q", email, series, su.Series())
-	}
-	if len(su.Models()) > 0 && !strutil.ListContains(su.Models(), model) {
-		return "", nil, fmt.Errorf(errorPrefix+"%q not in models %q", model, su.Models())
-	}
-	if !su.ValidAt(time.Now()) {
-		return "", nil, fmt.Errorf(errorPrefix + "assertion not valid anymore")
-	}
-
-	gecos := fmt.Sprintf("%s,%s", email, su.Name())
-	opts := &osutil.AddUserOptions{
-		SSHKeys:             su.SSHKeys(),
-		Gecos:               gecos,
-		Password:            su.Password(),
-		ForcePasswordChange: su.ForcePasswordChange(),
-	}
-	return su.Username(), opts, nil
-}
-
-type postUserCreateData struct {
-	Email        string `json:"email"`
-	Sudoer       bool   `json:"sudoer"`
-	Known        bool   `json:"known"`
-	ForceManaged bool   `json:"force-managed"`
-}
-
-var userLookup = user.Lookup
-
-func setupLocalUser(st *state.State, username, email string) error {
-	user, err := userLookup(username)
-	if err != nil {
-		return fmt.Errorf("cannot lookup user %q: %s", username, err)
-	}
-	uid, gid, err := osutil.UidGid(user)
-	if err != nil {
-		return err
-	}
-	authDataFn := filepath.Join(user.HomeDir, ".snap", "auth.json")
-	if err := osutil.MkdirAllChown(filepath.Dir(authDataFn), 0700, uid, gid); err != nil {
-		return err
-	}
-
-	// setup new user, local-only
-	st.Lock()
-	authUser, err := auth.NewUser(st, username, email, "", nil)
-	st.Unlock()
-	if err != nil {
-		return fmt.Errorf("cannot persist authentication details: %v", err)
-	}
-	// store macaroon auth, user's ID, email and username in auth.json in
-	// the new users home dir
-	outStr, err := json.Marshal(struct {
-		ID       int    `json:"id"`
-		Username string `json:"username"`
-		Email    string `json:"email"`
-		Macaroon string `json:"macaroon"`
-	}{
-		ID:       authUser.ID,
-		Username: authUser.Username,
-		Email:    authUser.Email,
-		Macaroon: authUser.Macaroon,
-	})
-	if err != nil {
-		return fmt.Errorf("cannot marshal auth data: %s", err)
-	}
-	if err := osutil.AtomicWriteFileChown(authDataFn, []byte(outStr), 0600, 0, uid, gid); err != nil {
-		return fmt.Errorf("cannot write auth file %q: %s", authDataFn, err)
-	}
-
-	return nil
-}
-
-func postCreateUser(c *Command, r *http.Request, user *auth.UserState) Response {
-	_, uid, _, err := postCreateUserUcrednetGet(r.RemoteAddr)
-	if err != nil {
-		return BadRequest("cannot get ucrednet uid: %v", err)
-	}
-	if uid != 0 {
-		return BadRequest("cannot use create-user as non-root")
-	}
-
-	var createData postUserCreateData
-
-	decoder := json.NewDecoder(r.Body)
-	if err := decoder.Decode(&createData); err != nil {
-		return BadRequest("cannot decode create-user data from request body: %v", err)
-	}
-
-	// verify request
-	st := c.d.overlord.State()
-	st.Lock()
-	users, err := auth.Users(st)
-	st.Unlock()
-	if err != nil {
-		return InternalError("cannot get user count: %s", err)
-	}
-
-	if !createData.ForceManaged {
-		if len(users) > 0 {
-			return BadRequest("cannot create user: device already managed")
-		}
-		if release.OnClassic {
-			return BadRequest("cannot create user: device is a classic system")
-		}
-	}
-
-	// special case: the user requested the creation of all known
-	// system-users
-	if createData.Email == "" && createData.Known {
-		return createAllKnownSystemUsers(c.d.overlord.State(), &createData)
-	}
-	if createData.Email == "" {
-		return BadRequest("cannot create user: 'email' field is empty")
-	}
-
-	var username string
-	var opts *osutil.AddUserOptions
-	if createData.Known {
-		username, opts, err = getUserDetailsFromAssertion(st, createData.Email)
-	} else {
-		username, opts, err = getUserDetailsFromStore(getStore(c), createData.Email)
-	}
-	if err != nil {
-		return BadRequest("%s", err)
-	}
-
-	// FIXME: duplicated code
-	opts.Sudoer = createData.Sudoer
-	opts.ExtraUsers = !release.OnClassic
-
-	if err := osutilAddUser(username, opts); err != nil {
-		return BadRequest("cannot create user %s: %s", username, err)
-	}
-
-	if err := setupLocalUser(c.d.overlord.State(), username, createData.Email); err != nil {
-		return InternalError("%s", err)
-	}
-
-	return SyncResponse(&userResponseData{
-		Username: username,
-		SSHKeys:  opts.SSHKeys,
-	}, nil)
-}
-
 func convertBuyError(err error) Response {
 	switch err {
 	case nil:
@@ -2486,34 +2128,6 @@ func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 	return SyncResponse(result, nil)
 }
 
-func getUsers(c *Command, r *http.Request, user *auth.UserState) Response {
-	_, uid, _, err := postCreateUserUcrednetGet(r.RemoteAddr)
-	if err != nil {
-		return BadRequest("cannot get ucrednet uid: %v", err)
-	}
-	if uid != 0 {
-		return BadRequest("cannot get users as non-root")
-	}
-
-	st := c.d.overlord.State()
-	st.Lock()
-	users, err := auth.Users(st)
-	st.Unlock()
-	if err != nil {
-		return InternalError("cannot get users: %s", err)
-	}
-
-	resp := make([]userResponseData, len(users))
-	for i, u := range users {
-		resp[i] = userResponseData{
-			Username: u.Username,
-			Email:    u.Email,
-			ID:       u.ID,
-		}
-	}
-	return SyncResponse(resp, nil)
-}
-
 // aliasAction is an action performed on aliases
 type aliasAction struct {
 	Action string `json:"action"`
@@ -2704,7 +2318,7 @@ func getLogs(c *Command, r *http.Request, user *auth.UserState) Response {
 		serviceNames[i] = appInfo.ServiceName()
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, progress.Null)
+	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, progress.Null)
 	reader, err := sysd.LogReader(serviceNames, n, follow)
 	if err != nil {
 		return InternalError("cannot get logs: %v", err)
