@@ -20,20 +20,18 @@
 package daemon
 
 import (
+	"context"
 	"fmt"
 	"net"
 	"net/http"
 	"os"
 	"os/exec"
 	"os/signal"
-	"runtime"
 	"strconv"
 	"strings"
 	"sync"
-	unix "syscall"
 	"time"
 
-	"github.com/coreos/go-systemd/activation"
 	"github.com/gorilla/mux"
 	"gopkg.in/tomb.v2"
 
@@ -42,12 +40,14 @@ import (
 	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/netutil"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/standby"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/polkit"
+	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/systemd"
 )
 
@@ -59,10 +59,11 @@ var systemdSdNotify = systemd.SdNotify
 type Daemon struct {
 	Version         string
 	overlord        *overlord.Overlord
+	state           *state.State
 	snapdListener   net.Listener
-	snapdServe      *shutdownServer
 	snapListener    net.Listener
-	snapServe       *shutdownServer
+	connTracker     *connTracker
+	serve           *http.Server
 	tomb            tomb.Tomb
 	router          *mux.Router
 	standbyOpinions *standby.StandbyOpinions
@@ -75,6 +76,8 @@ type Daemon struct {
 	// degradedErr is set when the daemon is in degraded mode
 	degradedErr error
 
+	expectedRebootDidNotHappen bool
+
 	mu sync.Mutex
 }
 
@@ -83,7 +86,8 @@ type ResponseFunc func(*Command, *http.Request, *auth.UserState) Response
 
 // A Command routes a request to an individual per-verb ResponseFUnc
 type Command struct {
-	Path string
+	Path       string
+	PathPrefix string
 	//
 	GET    ResponseFunc
 	PUT    ResponseFunc
@@ -95,6 +99,8 @@ type Command struct {
 	UserOK bool
 	// is this path accessible on the snapd-snap socket?
 	SnapOK bool
+	// this path is only accessible to root
+	RootOnly bool
 
 	// can polkit grant access? set to polkit action ID if so
 	PolkitOK string
@@ -115,17 +121,23 @@ var polkitCheckAuthorization = polkit.CheckAuthorization
 
 // canAccess checks the following properties:
 //
-// - if a user is logged in (via `snap login`) everything is allowed
 // - if the user is `root` everything is allowed
-// - POST/PUT/DELETE all require `snap login` or `root`
+// - if a user is logged in (via `snap login`) and the command doesn't have RootOnly, everything is allowed
+// - POST/PUT/DELETE all require `root`, or just `snap login` if not RootOnly
 //
 // Otherwise for GET requests the following parameters are honored:
 // - GuestOK: anyone can access GET
 // - UserOK: any uid on the local system can access GET
+// - RootOnly: only root can access this
 // - SnapOK: a snap can access this via `snapctl`
 func (c *Command) canAccess(r *http.Request, user *auth.UserState) accessResult {
-	if user != nil {
-		// Authenticated users do anything for now.
+	if c.RootOnly && (c.UserOK || c.GuestOK || c.SnapOK) {
+		// programming error
+		logger.Panicf("Command can't have RootOnly together with any *OK flag")
+	}
+
+	if user != nil && !c.RootOnly {
+		// Authenticated users do anything not requiring explicit root.
 		return accessOK
 	}
 
@@ -148,7 +160,8 @@ func (c *Command) canAccess(r *http.Request, user *auth.UserState) accessResult
 		return accessUnauthorized
 	}
 
-	if r.Method == "GET" {
+	// the !RootOnly check is redundant, but belt-and-suspenders
+	if r.Method == "GET" && !c.RootOnly {
 		// Guest and user access restricted to GET requests
 		if c.GuestOK {
 			return accessOK
@@ -169,6 +182,10 @@ func (c *Command) canAccess(r *http.Request, user *auth.UserState) accessResult
 		return accessOK
 	}
 
+	if c.RootOnly {
+		return accessUnauthorized
+	}
+
 	if c.PolkitOK != "" {
 		var flags polkit.CheckFlags
 		allowHeader := r.Header.Get(client.AllowInteractionHeader)
@@ -196,7 +213,7 @@ func (c *Command) canAccess(r *http.Request, user *auth.UserState) accessResult
 }
 
 func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
-	st := c.d.overlord.State()
+	st := c.d.state
 	st.Lock()
 	// TODO Look at the error and fail if there's an attempt to authenticate with invalid data.
 	user, _ := UserFromRequest(st, r)
@@ -222,6 +239,9 @@ func (c *Command) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
+	ctx := store.WithClientUserAgent(r.Context(), r)
+	r = r.WithContext(ctx)
+
 	var rspf ResponseFunc
 	var rsp = MethodNotAllowed("method %q not allowed", r.Method)
 
@@ -298,78 +318,25 @@ func logit(handler http.Handler) http.Handler {
 	})
 }
 
-// getListener tries to get a listener for the given socket path from
-// the listener map, and if it fails it tries to set it up directly.
-func getListener(socketPath string, listenerMap map[string]net.Listener) (net.Listener, error) {
-	if listener, ok := listenerMap[socketPath]; ok {
-		return listener, nil
-	}
-
-	if c, err := net.Dial("unix", socketPath); err == nil {
-		c.Close()
-		return nil, fmt.Errorf("socket %q already in use", socketPath)
-	}
-
-	if err := os.Remove(socketPath); err != nil && !os.IsNotExist(err) {
-		return nil, err
-	}
-
-	address, err := net.ResolveUnixAddr("unix", socketPath)
-	if err != nil {
-		return nil, err
-	}
-
-	runtime.LockOSThread()
-	oldmask := unix.Umask(0111)
-	listener, err := net.ListenUnix("unix", address)
-	unix.Umask(oldmask)
-	runtime.UnlockOSThread()
-	if err != nil {
-		return nil, err
-	}
-
-	logger.Debugf("socket %q was not activated; listening", socketPath)
-
-	return listener, nil
-}
-
-// activationListeners builds a map of addresses to listeners that were passed
-// during systemd activation
-func activationListeners() (lns map[string]net.Listener, err error) {
-	// pass false to keep LISTEN_* environment variables passed by systemd
-	files := activation.Files(false)
-	lns = make(map[string]net.Listener, len(files))
-
-	for _, f := range files {
-		ln, err := net.FileListener(f)
-		if err != nil {
-			return nil, err
-		}
-		addr := ln.Addr().String()
-		lns[addr] = ln
-	}
-	return lns, nil
-}
-
 // Init sets up the Daemon's internal workings.
 // Don't call more than once.
 func (d *Daemon) Init() error {
-	listenerMap, err := activationListeners()
+	listenerMap, err := netutil.ActivationListeners()
 	if err != nil {
 		return err
 	}
 
 	// The SnapdSocket is required-- without it, die.
-	if listener, err := getListener(dirs.SnapdSocket, listenerMap); err == nil {
-		d.snapdListener = &ucrednetListener{listener}
+	if listener, err := netutil.GetListener(dirs.SnapdSocket, listenerMap); err == nil {
+		d.snapdListener = &ucrednetListener{Listener: listener}
 	} else {
 		return fmt.Errorf("when trying to listen on %s: %v", dirs.SnapdSocket, err)
 	}
 
-	if listener, err := getListener(dirs.SnapSocket, listenerMap); err == nil {
+	if listener, err := netutil.GetListener(dirs.SnapSocket, listenerMap); err == nil {
 		// This listener may also be nil if that socket wasn't among
 		// the listeners, so check it before using it.
-		d.snapListener = &ucrednetListener{listener}
+		d.snapListener = &ucrednetListener{Listener: listener}
 	} else {
 		logger.Debugf("cannot get listener for %q: %v", dirs.SnapSocket, err)
 	}
@@ -400,7 +367,11 @@ func (d *Daemon) addRoutes() {
 
 	for _, c := range api {
 		c.d = d
-		d.router.Handle(c.Path, c).Name(c.Path)
+		if c.PathPrefix == "" {
+			d.router.Handle(c.Path, c).Name(c.Path)
+		} else {
+			d.router.PathPrefix(c.PathPrefix).Handler(c).Name(c.PathPrefix)
+		}
 	}
 
 	// also maybe add a /favicon.ico handler...
@@ -409,100 +380,36 @@ func (d *Daemon) addRoutes() {
 }
 
 var (
-	shutdownTimeout = 5 * time.Second
+	shutdownTimeout = 25 * time.Second
 )
 
-// shutdownServer supplements a http.Server with graceful shutdown.
-// TODO: with go1.8 http.Server itself grows a graceful Shutdown method
-type shutdownServer struct {
-	l       net.Listener
-	httpSrv *http.Server
-
-	mu           sync.Mutex
-	conns        map[net.Conn]http.ConnState
-	shuttingDown bool
+type connTracker struct {
+	mu    sync.Mutex
+	conns map[net.Conn]struct{}
 }
 
-func newShutdownServer(l net.Listener, h http.Handler) *shutdownServer {
-	srv := &http.Server{
-		Handler: h,
-	}
-	ssrv := &shutdownServer{
-		l:       l,
-		httpSrv: srv,
-		conns:   make(map[net.Conn]http.ConnState),
-	}
-	srv.ConnState = ssrv.trackConn
-	return ssrv
+func (ct *connTracker) CanStandby() bool {
+	ct.mu.Lock()
+	defer ct.mu.Unlock()
+
+	return len(ct.conns) == 0
 }
 
-func (srv *shutdownServer) Serve() error {
-	return srv.httpSrv.Serve(srv.l)
-}
-
-func (srv *shutdownServer) CanStandby() bool {
-	srv.mu.Lock()
-	defer srv.mu.Unlock()
-
-	for _, state := range srv.conns {
-		if state != http.StateIdle {
-			return false
-		}
-	}
-	return true
-}
-
-func (srv *shutdownServer) trackConn(conn net.Conn, state http.ConnState) {
-	srv.mu.Lock()
-	defer srv.mu.Unlock()
+func (ct *connTracker) trackConn(conn net.Conn, state http.ConnState) {
+	ct.mu.Lock()
+	defer ct.mu.Unlock()
 	// we ignore hijacked connections, if we do things with websockets
 	// we'll need custom shutdown handling for them
-	if state == http.StateClosed || state == http.StateHijacked {
-		delete(srv.conns, conn)
-		return
+	if state == http.StateNew || state == http.StateActive {
+		ct.conns[conn] = struct{}{}
+	} else {
+		delete(ct.conns, conn)
 	}
-	if srv.shuttingDown && state == http.StateIdle {
-		conn.Close()
-		delete(srv.conns, conn)
-		return
-	}
-	srv.conns[conn] = state
-}
-
-func (srv *shutdownServer) finishShutdown() error {
-	toutC := time.After(shutdownTimeout)
-
-	srv.mu.Lock()
-	defer srv.mu.Unlock()
-
-	srv.shuttingDown = true
-	for conn, state := range srv.conns {
-		if state == http.StateIdle {
-			conn.Close()
-			delete(srv.conns, conn)
-		}
-	}
-
-	doWait := true
-	for doWait {
-		if len(srv.conns) == 0 {
-			return nil
-		}
-		srv.mu.Unlock()
-		select {
-		case <-time.After(200 * time.Millisecond):
-		case <-toutC:
-			doWait = false
-		}
-		srv.mu.Lock()
-	}
-	return fmt.Errorf("cannot gracefully finish, still active connections on %v after %v", srv.l.Addr(), shutdownTimeout)
 }
 
 func (d *Daemon) initStandbyHandling() {
-	d.standbyOpinions = standby.New(d.overlord.State())
-	d.standbyOpinions.AddOpinion(d.snapdServe)
-	d.standbyOpinions.AddOpinion(d.snapServe)
+	d.standbyOpinions = standby.New(d.state)
+	d.standbyOpinions.AddOpinion(d.connTracker)
 	d.standbyOpinions.AddOpinion(d.overlord)
 	d.standbyOpinions.AddOpinion(d.overlord.SnapManager())
 	d.standbyOpinions.AddOpinion(d.overlord.DeviceManager())
@@ -510,37 +417,28 @@ func (d *Daemon) initStandbyHandling() {
 }
 
 // Start the Daemon
-func (d *Daemon) Start() {
-	// die when asked to restart (systemd should get us back up!)
-	d.overlord.SetRestartHandler(func(t state.RestartType) {
-		switch t {
-		case state.RestartDaemon:
-			d.tomb.Kill(nil)
-		case state.RestartSystem:
-			// try to schedule a fallback slow reboot already here
-			// in case we get stuck shutting down
-			if err := reboot(rebootWaitTimeout); err != nil {
-				logger.Noticef("%s", err)
-			}
+func (d *Daemon) Start() error {
+	if d.expectedRebootDidNotHappen {
+		// we need to schedule and wait for a system restart
+		d.tomb.Kill(nil)
+		// avoid systemd killing us again while we wait
+		systemdSdNotify("READY=1")
+		return nil
+	}
+	if d.overlord == nil {
+		panic("internal error: no Overlord")
+	}
 
-			d.mu.Lock()
-			defer d.mu.Unlock()
-			// remember we need to restart the system
-			d.restartSystem = true
-			d.tomb.Kill(nil)
-		case state.RestartSocket:
-			d.restartSocket = true
-			d.tomb.Kill(nil)
-		default:
-			logger.Noticef("internal error: restart handler called with unknown restart type: %v", t)
-			d.tomb.Kill(nil)
-		}
-	})
+	// now perform expensive overlord/manages initiliazation
+	if err := d.overlord.StartUp(); err != nil {
+		return err
+	}
 
-	if d.snapListener != nil {
-		d.snapServe = newShutdownServer(d.snapListener, logit(d.router))
+	d.connTracker = &connTracker{conns: make(map[net.Conn]struct{})}
+	d.serve = &http.Server{
+		Handler:   logit(d.router),
+		ConnState: d.connTracker.trackConn,
 	}
-	d.snapdServe = newShutdownServer(d.snapdListener, logit(d.router))
 
 	// enable standby handling
 	d.initStandbyHandling()
@@ -551,7 +449,7 @@ func (d *Daemon) Start() {
 	d.tomb.Go(func() error {
 		if d.snapListener != nil {
 			d.tomb.Go(func() error {
-				if err := d.snapServe.Serve(); err != nil && d.tomb.Err() == tomb.ErrStillAlive {
+				if err := d.serve.Serve(d.snapListener); err != http.ErrServerClosed && d.tomb.Err() == tomb.ErrStillAlive {
 					return err
 				}
 
@@ -559,7 +457,7 @@ func (d *Daemon) Start() {
 			})
 		}
 
-		if err := d.snapdServe.Serve(); err != nil && d.tomb.Err() == tomb.ErrStillAlive {
+		if err := d.serve.Serve(d.snapdListener); err != http.ErrServerClosed && d.tomb.Err() == tomb.ErrStillAlive {
 			return err
 		}
 
@@ -568,31 +466,52 @@ func (d *Daemon) Start() {
 
 	// notify systemd that we are ready
 	systemdSdNotify("READY=1")
-}
-
-var shutdownMsg = i18n.G("reboot scheduled to update the system")
-
-func rebootImpl(rebootDelay time.Duration) error {
-	if rebootDelay < 0 {
-		rebootDelay = 0
-	}
-	mins := int64((rebootDelay + time.Minute - 1) / time.Minute)
-	cmd := exec.Command("shutdown", "-r", fmt.Sprintf("+%d", mins), shutdownMsg)
-	if out, err := cmd.CombinedOutput(); err != nil {
-		return osutil.OutputErr(out, err)
-	}
 	return nil
 }
 
-var reboot = rebootImpl
+// HandleRestart implements overlord.RestartBehavior.
+func (d *Daemon) HandleRestart(t state.RestartType) {
+	// die when asked to restart (systemd should get us back up!) etc
+	switch t {
+	case state.RestartDaemon:
+	case state.RestartSystem:
+		// try to schedule a fallback slow reboot already here
+		// in case we get stuck shutting down
+		if err := reboot(rebootWaitTimeout); err != nil {
+			logger.Noticef("%s", err)
+		}
+
+		d.mu.Lock()
+		defer d.mu.Unlock()
+		// remember we need to restart the system
+		d.restartSystem = true
+	case state.RestartSocket:
+		d.mu.Lock()
+		defer d.mu.Unlock()
+		d.restartSocket = true
+	default:
+		logger.Noticef("internal error: restart handler called with unknown restart type: %v", t)
+	}
+	d.tomb.Kill(nil)
+}
 
 var (
-	rebootNoticeWait  = 3 * time.Second
-	rebootWaitTimeout = 10 * time.Minute
+	rebootNoticeWait       = 3 * time.Second
+	rebootWaitTimeout      = 10 * time.Minute
+	rebootRetryWaitTimeout = 5 * time.Minute
+	rebootMaxTentatives    = 3
 )
 
 // Stop shuts down the Daemon
 func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
+	// we need to schedule/wait for a system restart again
+	if d.expectedRebootDidNotHappen {
+		return d.doReboot(sigCh, rebootRetryWaitTimeout)
+	}
+	if d.overlord == nil {
+		return fmt.Errorf("internal error: no Overlord")
+	}
+
 	d.tomb.Kill(nil)
 
 	d.mu.Lock()
@@ -607,7 +526,7 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 		// stop running hooks first
 		// and do it more gracefully if we are restarting
 		hookMgr := d.overlord.HookManager()
-		if ok, _ := d.overlord.State().Restarting(); ok {
+		if ok, _ := d.state.Restarting(); ok {
 			logger.Noticef("gracefully waiting for running hooks")
 			hookMgr.GracefullyWaitRunningHooks()
 			logger.Noticef("done waiting for running hooks")
@@ -621,10 +540,12 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 		time.Sleep(rebootNoticeWait)
 	}
 
-	d.tomb.Kill(d.snapdServe.finishShutdown())
-	if d.snapListener != nil {
-		d.tomb.Kill(d.snapServe.finishShutdown())
-	}
+	// We're using the background context here because the tomb's
+	// context will likely already have been cancelled when we are
+	// called.
+	ctx, cancel := context.WithTimeout(context.Background(), shutdownTimeout)
+	d.tomb.Kill(d.serve.Shutdown(ctx))
+	cancel()
 
 	if !restartSystem {
 		// tell systemd that we are stopping
@@ -649,36 +570,21 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 
 	err := d.tomb.Wait()
 	if err != nil {
-		return err
+		// do not stop the shutdown even if the tomb errors
+		// because we already scheduled a slow shutdown and
+		// exiting here will just restart snapd (via systemd)
+		// which will lead to confusing results.
+		if restartSystem {
+			logger.Noticef("WARNING: cannot stop daemon: %v", err)
+		} else {
+			return err
+		}
 	}
 
 	if restartSystem {
-		// ask for shutdown and wait for it to happen.
-		// if we exit snapd will be restared by systemd
-		rebootDelay := 1 * time.Minute
-		ovr := os.Getenv("SNAPD_REBOOT_DELAY") // for tests
-		if ovr != "" {
-			d, err := time.ParseDuration(ovr)
-			if err == nil {
-				rebootDelay = d
-			}
-		}
-		if err := reboot(rebootDelay); err != nil {
-			return err
-		}
-		// wait for reboot to happen
-		logger.Noticef("Waiting for system reboot")
-		if sigCh != nil {
-			signal.Stop(sigCh)
-			if len(sigCh) > 0 {
-				// a signal arrived in between
-				return nil
-			}
-			close(sigCh)
-		}
-		time.Sleep(rebootWaitTimeout)
-		return fmt.Errorf("expected reboot did not happen")
+		return d.doReboot(sigCh, rebootWaitTimeout)
 	}
+
 	if d.restartSocket {
 		return ErrRestartSocket
 	}
@@ -686,16 +592,127 @@ func (d *Daemon) Stop(sigCh chan<- os.Signal) error {
 	return nil
 }
 
+func (d *Daemon) rebootDelay() (time.Duration, error) {
+	d.state.Lock()
+	defer d.state.Unlock()
+	now := time.Now()
+	// see whether a reboot had already been scheduled
+	var rebootAt time.Time
+	err := d.state.Get("daemon-system-restart-at", &rebootAt)
+	if err != nil && err != state.ErrNoState {
+		return 0, err
+	}
+	rebootDelay := 1 * time.Minute
+	if err == nil {
+		rebootDelay = rebootAt.Sub(now)
+	} else {
+		ovr := os.Getenv("SNAPD_REBOOT_DELAY") // for tests
+		if ovr != "" {
+			d, err := time.ParseDuration(ovr)
+			if err == nil {
+				rebootDelay = d
+			}
+		}
+		rebootAt = now.Add(rebootDelay)
+		d.state.Set("daemon-system-restart-at", rebootAt)
+	}
+	return rebootDelay, nil
+}
+
+func (d *Daemon) doReboot(sigCh chan<- os.Signal, waitTimeout time.Duration) error {
+	rebootDelay, err := d.rebootDelay()
+	if err != nil {
+		return err
+	}
+	// ask for shutdown and wait for it to happen.
+	// if we exit snapd will be restared by systemd
+	if err := reboot(rebootDelay); err != nil {
+		return err
+	}
+	// wait for reboot to happen
+	logger.Noticef("Waiting for system reboot")
+	if sigCh != nil {
+		signal.Stop(sigCh)
+		if len(sigCh) > 0 {
+			// a signal arrived in between
+			return nil
+		}
+		close(sigCh)
+	}
+	time.Sleep(waitTimeout)
+	return fmt.Errorf("expected reboot did not happen")
+}
+
+var shutdownMsg = i18n.G("reboot scheduled to update the system")
+
+func rebootImpl(rebootDelay time.Duration) error {
+	if rebootDelay < 0 {
+		rebootDelay = 0
+	}
+	mins := int64(rebootDelay / time.Minute)
+	cmd := exec.Command("shutdown", "-r", fmt.Sprintf("+%d", mins), shutdownMsg)
+	if out, err := cmd.CombinedOutput(); err != nil {
+		return osutil.OutputErr(out, err)
+	}
+	return nil
+}
+
+var reboot = rebootImpl
+
 // Dying is a tomb-ish thing
 func (d *Daemon) Dying() <-chan struct{} {
 	return d.tomb.Dying()
 }
 
+func clearReboot(st *state.State) {
+	st.Set("daemon-system-restart-at", nil)
+	st.Set("daemon-system-restart-tentative", nil)
+}
+
+// RebootAsExpected implements part of overlord.RestartBehavior.
+func (d *Daemon) RebootAsExpected(st *state.State) error {
+	clearReboot(st)
+	return nil
+}
+
+// RebootDidNotHappen implements part of overlord.RestartBehavior.
+func (d *Daemon) RebootDidNotHappen(st *state.State) error {
+	var nTentative int
+	err := st.Get("daemon-system-restart-tentative", &nTentative)
+	if err != nil && err != state.ErrNoState {
+		return err
+	}
+	nTentative++
+	if nTentative > rebootMaxTentatives {
+		// giving up, proceed normally, some in-progress refresh
+		// might get rolled back!!
+		st.ClearReboot()
+		clearReboot(st)
+		logger.Noticef("snapd was restarted while a system restart was expected, snapd retried to schedule and waited again for a system restart %d times and is giving up", rebootMaxTentatives)
+		return nil
+	}
+	st.Set("daemon-system-restart-tentative", nTentative)
+	d.state = st
+	logger.Noticef("snapd was restarted while a system restart was expected, snapd will try to schedule and wait for a system restart again (tenative %d/%d)", nTentative, rebootMaxTentatives)
+	return state.ErrExpectedReboot
+}
+
 // New Daemon
 func New() (*Daemon, error) {
-	ovld, err := overlord.New()
+	d := &Daemon{}
+	ovld, err := overlord.New(d)
+	if err == state.ErrExpectedReboot {
+		// we proceed without overlord until we reach Stop
+		// where we will schedule and wait again for a system restart.
+		// ATM we cannot do that in New because we need to satisfy
+		// systemd notify mechanisms.
+		d.expectedRebootDidNotHappen = true
+		return d, nil
+	}
 	if err != nil {
 		return nil, err
 	}
-	return &Daemon{overlord: ovld}, nil
+	d.overlord = ovld
+	d.state = ovld.State()
+	return d, nil
 }
