@@ -246,11 +246,40 @@ func makeErrorResponder(status int) errorResponder {
 	}
 }
 
-// A FileResponse 's ServeHTTP method serves the file
-type FileResponse string
+// A FileStream ServeHTTP method streams the snap
+type fileStream struct {
+	SnapName string
+	Info     snap.DownloadInfo
+	stream   io.ReadCloser
+}
 
 // ServeHTTP from the Response interface
-func (f FileResponse) ServeHTTP(w http.ResponseWriter, r *http.Request) {
+func (s fileStream) ServeHTTP(w http.ResponseWriter, _ *http.Request) {
+	hdr := w.Header()
+	hdr.Set("Content-Type", "application/octet-stream")
+	snapname := fmt.Sprintf("attachment; filename=%s", s.SnapName)
+	hdr.Set("Content-Disposition", snapname)
+
+	size := fmt.Sprintf("%d", s.Info.Size)
+	hdr.Set("Content-Length", size)
+
+	defer s.stream.Close()
+	bytesCopied, err := io.Copy(w, s.stream)
+	if err != nil {
+		logger.Noticef("cannot copy snap %s (%#v) to the stream: %v", s.SnapName, s.Info, err)
+		http.Error(w, err.Error(), 500)
+	}
+	if bytesCopied != s.Info.Size {
+		logger.Noticef("cannot copy snap %s (%#v) to the stream: bytes copied=%d, expected=%d", s.SnapName, s.Info, bytesCopied, s.Info.Size)
+		http.Error(w, io.EOF.Error(), 502)
+	}
+}
+
+// A fileResponse 's ServeHTTP method serves the file
+type fileResponse string
+
+// ServeHTTP from the Response interface
+func (f fileResponse) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	filename := fmt.Sprintf("attachment; filename=%s", filepath.Base(string(f)))
 	w.Header().Add("Content-Disposition", filename)
 	http.ServeFile(w, r, string(f))
