@@ -20,6 +20,7 @@
 package image
 
 import (
+	"bytes"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -348,7 +349,7 @@ func acquireSnap(tsto *ToolingStore, name string, dlOpts *DownloadOptions, local
 		}
 		return dst, info, nil
 	}
-	return tsto.DownloadSnap(name, snap.R(0), dlOpts)
+	return tsto.DownloadSnap(name, *dlOpts)
 }
 
 type addingFetcher struct {
@@ -411,10 +412,16 @@ func neededDefaultProviders(info *snap.Info) (cps []string) {
 // hasBase checks if the given snap has a base in the given localInfos and
 // snaps. If not an error is returned.
 func hasBase(snap *snap.Info, local *localInfos, snaps []string) error {
-	// snap needs no base: nothing to do
+	// snap needs no base (or it simply needs core which is never listed explicitly): nothing to do
 	if snap.Base == "" {
 		return nil
 	}
+
+	// snap explicitly listed as not needing a base snap (e.g. a content-only snap)
+	if snap.Base == "none" {
+		return nil
+	}
+
 	// core provides everything that core16 needs
 	if snap.Base == "core16" && local.hasName(snaps, "core") {
 		return nil
@@ -554,7 +561,7 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options, local *l
 		// Sanity check, note that we could support this case
 		// if we have a use-case but it requires changes in the
 		// devicestate/firstboot.go ordering code.
-		if info.Type == snap.TypeGadget && info.Base != model.Base() {
+		if info.GetType() == snap.TypeGadget && info.Base != model.Base() {
 			return fmt.Errorf("cannot use gadget snap because its base %q is different from model base %q", info.Base, model.Base())
 		}
 		if err := hasBase(info, local, snaps); err != nil {
@@ -563,12 +570,13 @@ func setupSeed(tsto *ToolingStore, model *asserts.Model, opts *Options, local *l
 		// warn about missing default providers
 		for _, dp := range neededDefaultProviders(info) {
 			if !local.hasName(snaps, dp) {
-				fmt.Fprintf(Stderr, "WARNING: the default content provider %q requested by snap %q is not getting installed.", dp, info.InstanceName())
+				// TODO: have a way to ignore this issue on a snap by snap basis?
+				return fmt.Errorf("cannot use snap %q without its default content provider %q being added explicitly", info.InstanceName(), dp)
 			}
 		}
 
 		seen[name] = true
-		typ := info.Type
+		typ := info.GetType()
 
 		needsClassic := info.NeedsClassic()
 		if needsClassic && !opts.Classic {
@@ -742,7 +750,7 @@ func setBootvars(downloadedSnapsInfoForBootConfig map[string]*snap.Info, model *
 			}
 			return fmt.Errorf("cannot get download info for snap %s, available infos: %v", fn, keys)
 		}
-		switch info.Type {
+		switch info.GetType() {
 		case snap.TypeOS, snap.TypeBase:
 			bootvar = "snap_core"
 		case snap.TypeKernel:
@@ -780,3 +788,67 @@ func copyLocalSnapFile(snapPath, targetDir string, info *snap.Info) (dstPath str
 	dst := filepath.Join(targetDir, filepath.Base(info.MountFile()))
 	return dst, osutil.CopyFile(snapPath, dst, 0)
 }
+
+func ValidateSeed(seedFile string) error {
+	seed, err := snap.ReadSeedYaml(seedFile)
+	if err != nil {
+		return err
+	}
+
+	var errs []error
+	// read the snaps info
+	snapInfos := make(map[string]*snap.Info)
+	for _, seedSnap := range seed.Snaps {
+		fn := filepath.Join(filepath.Dir(seedFile), "snaps", seedSnap.File)
+		snapf, err := snap.Open(fn)
+		if err != nil {
+			errs = append(errs, err)
+		} else {
+			info, err := snap.ReadInfoFromSnapFile(snapf, nil)
+			if err != nil {
+				errs = append(errs, fmt.Errorf("cannot use snap %s: %v", fn, err))
+			} else {
+				snapInfos[info.InstanceName()] = info
+			}
+		}
+	}
+
+	// ensure we have either "core" or "snapd"
+	_, haveCore := snapInfos["core"]
+	_, haveSnapd := snapInfos["snapd"]
+	if !(haveCore || haveSnapd) {
+		errs = append(errs, fmt.Errorf("the core or snapd snap must be part of the seed"))
+	}
+
+	// check that all bases/default-providers are part of the seed
+	for _, info := range snapInfos {
+		// ensure base is available
+		if info.Base != "" && info.Base != "none" {
+			if _, ok := snapInfos[info.Base]; !ok {
+				errs = append(errs, fmt.Errorf("cannot use snap %q: base %q is missing", info.InstanceName(), info.Base))
+			}
+		}
+		// ensure core is available
+		if info.Base == "" && info.SnapType == snap.TypeApp && info.InstanceName() != "snapd" {
+			if _, ok := snapInfos["core"]; !ok {
+				errs = append(errs, fmt.Errorf(`cannot use snap %q: required snap "core" missing`, info.InstanceName()))
+			}
+		}
+		// ensure default-providers are available
+		for _, dp := range neededDefaultProviders(info) {
+			if _, ok := snapInfos[dp]; !ok {
+				errs = append(errs, fmt.Errorf("cannot use snap %q: default provider %q is missing", info.InstanceName(), dp))
+			}
+		}
+	}
+
+	if errs != nil {
+		var buf bytes.Buffer
+		for _, err := range errs {
+			fmt.Fprintf(&buf, "\n- %s", err)
+		}
+		return fmt.Errorf("cannot validate seed:%s", buf.Bytes())
+	}
+
+	return nil
+}
