@@ -25,6 +25,7 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/seccomp"
+	"github.com/snapcore/snapd/interfaces/udev"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 )
@@ -45,8 +46,17 @@ const dockerSupportBaseDeclarationSlots = `
     deny-auto-connection: true
 `
 
+const dockerSupportConnectedPlugAppArmorCore = `
+# These accesses are necessary for Ubuntu Core 16 and 18, likely due to the
+# version of apparmor or the kernel which doesn't resolve the upper layer of an
+# overlayfs mount correctly the accesses show up as runc trying to read from
+# /system-data/var/snap/docker/common/var-lib-docker/overlay2/$SHA/diff/
+/system-data/var/snap/{@{SNAP_NAME},@{SNAP_INSTANCE_NAME}}/common/{,**/} rwl,
+/system-data/var/snap/{@{SNAP_NAME},@{SNAP_INSTANCE_NAME}}/@{SNAP_REVISION}/{,**/} rwl,
+`
+
 const dockerSupportConnectedPlugAppArmor = `
-# Description: allow operating as the Docker daemon. This policy is
+# Description: allow operating as the Docker daemon/containerd. This policy is
 # intentionally not restrictive and is here to help guard against programming
 # errors and not for security confinement. The Docker daemon by design requires
 # extensive access to the system and cannot be effectively confined against
@@ -54,15 +64,26 @@ const dockerSupportConnectedPlugAppArmor = `
 
 #include <abstractions/dbus-strict>
 
-# Allow sockets
+# Allow sockets/etc for docker
 /{,var/}run/docker.sock rw,
 /{,var/}run/docker/     rw,
 /{,var/}run/docker/**   mrwklix,
 /{,var/}run/runc/       rw,
 /{,var/}run/runc/**     mrwklix,
 
+# Allow sockets/etc for containerd
+/{,var/}run/containerd/{,runc/,runc/k8s.io/,runc/k8s.io/*/} rw,
+/{,var/}run/containerd/runc/k8s.io/*/** rwk,
+/{,var/}run/containerd/{io.containerd*/,io.containerd*/k8s.io/,io.containerd*/k8s.io/*/} rw,
+/{,var/}run/containerd/io.containerd*/*/** rwk,
+
+# Limit ipam-state to k8s
+/run/ipam-state/k8s-** rw,
+/run/ipam-state/k8s-*/lock k,
+
 # Socket for docker-container-shim
 unix (bind,listen) type=stream addr="@/containerd-shim/moby/*/shim.sock\x00",
+unix (bind,listen) type=stream addr="@/containerd-shim/k8s.io/*/shim.sock\x00",
 
 /{,var/}run/mount/utab r,
 
@@ -128,10 +149,17 @@ pivot_root,
 /sys/kernel/security/apparmor/{,**} r,
 
 # use 'privileged-containers: true' to support --security-opts
+
+# defaults for docker-default
 change_profile unsafe /** -> docker-default,
 signal (send) peer=docker-default,
 ptrace (read, trace) peer=docker-default,
 
+# defaults for containerd
+change_profile unsafe /** -> cri-containerd.apparmor.d,
+signal (send) peer=cri-containerd.apparmor.d,
+ptrace (read, trace) peer=cri-containerd.apparmor.d,
+
 # Graph (storage) driver bits
 /{dev,run}/shm/aufs.xino mrw,
 /proc/fs/aufs/plink_maint w,
@@ -140,14 +168,15 @@ ptrace (read, trace) peer=docker-default,
 #cf bug 1502785
 / r,
 
-# needed by runc for mitigation of CVE-2019-5736
-# For details see https://bugs.launchpad.net/apparmor/+bug/1820344
-/ ix,
-
 # recent versions of docker make a symlink from /dev/ptmx to /dev/pts/ptmx
 # and so to allow allocating a new shell we need this
 /dev/pts/ptmx rw,
 
+# needed by runc for mitigation of CVE-2019-5736
+# For details see https://bugs.launchpad.net/apparmor/+bug/1820344
+/ ix,
+/bin/runc ixr,
+/pause ixr,
 `
 
 const dockerSupportConnectedPlugSecComp = `
@@ -585,6 +614,12 @@ var (
 	parserFeatures = release.AppArmorParserFeatures
 )
 
+func (iface *dockerSupportInterface) UDevConnectedPlug(spec *udev.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	spec.SetControlsDeviceCgroup()
+
+	return nil
+}
+
 func (iface *dockerSupportInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	var privileged bool
 	_ = plug.Attr("privileged-containers", &privileged)
@@ -596,7 +631,10 @@ func (iface *dockerSupportInterface) AppArmorConnectedPlug(spec *apparmor.Specif
 	if privileged {
 		spec.AddSnippet(dockerSupportPrivilegedAppArmor)
 	}
-	spec.UsesPtraceTrace()
+	if !release.OnClassic {
+		spec.AddSnippet(dockerSupportConnectedPlugAppArmorCore)
+	}
+	spec.SetUsesPtraceTrace()
 	return nil
 }
 
