@@ -35,6 +35,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
+	seccomp_compiler "github.com/snapcore/snapd/sandbox/seccomp"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/testutil"
@@ -79,12 +80,12 @@ func (s *backendSuite) SetUpTest(c *C) {
 	c.Assert(err, IsNil)
 	s.snapSeccomp = testutil.MockCommand(c, snapSeccompPath, `
 if [ "$1" = "version-info" ]; then
-    echo "abcdef 1.2.3 1234abcd"
+    echo "abcdef 1.2.3 1234abcd -"
 fi`)
 
 	s.Backend.Initialize()
 	s.profileHeader = `# snap-seccomp version information:
-# abcdef 1.2.3 1234abcd
+# abcdef 1.2.3 1234abcd -
 `
 	// make sure initialize called version-info
 	c.Check(s.snapSeccomp.Calls(), DeepEquals, [][]string{
@@ -156,7 +157,7 @@ func (s *backendSuite) TestInstallingSnapWritesProfilesWithReexec(c *C) {
 	err := os.MkdirAll(filepath.Dir(snapSeccompOnCorePath), 0755)
 	c.Assert(err, IsNil)
 	snapSeccompOnCore := testutil.MockCommand(c, snapSeccompOnCorePath, `if [ "$1" = "version-info" ]; then
-echo "2345cdef 2.3.4 2345cdef"
+echo "2345cdef 2.3.4 2345cdef -"
 fi`)
 	defer snapSeccompOnCore.Restore()
 
@@ -179,7 +180,7 @@ fi`)
 	raw, err := ioutil.ReadFile(profile + ".src")
 	c.Assert(err, IsNil)
 	c.Assert(bytes.HasPrefix(raw, []byte(`# snap-seccomp version information:
-# 2345cdef 2.3.4 2345cdef
+# 2345cdef 2.3.4 2345cdef -
 `)), Equals, true)
 }
 
@@ -278,6 +279,7 @@ func (s *backendSuite) TestRealDefaultTemplateIsNormallyUsed(c *C) {
 		"# - create_module, init_module, finit_module, delete_module (kernel modules)\n",
 		"open\n",
 		"getuid\n",
+		"setresuid\n", // this is not random
 	} {
 		c.Assert(string(data), testutil.Contains, line)
 	}
@@ -472,6 +474,18 @@ func (s *backendSuite) TestSandboxFeatures(c *C) {
 	defer restore()
 
 	c.Assert(s.Backend.SandboxFeatures(), DeepEquals, []string{"kernel:foo", "kernel:bar", "bpf-argument-filtering"})
+
+	// change version reported by snap-seccomp
+	snapSeccomp := testutil.MockCommand(c, filepath.Join(dirs.DistroLibExecDir, "snap-seccomp"), `
+if [ "$1" = "version-info" ]; then
+    echo "abcdef 1.2.3 1234abcd bpf-actlog"
+fi`)
+	defer snapSeccomp.Restore()
+
+	// reload cached version info
+	err := s.Backend.Initialize()
+	c.Assert(err, IsNil)
+	c.Assert(s.Backend.SandboxFeatures(), DeepEquals, []string{"kernel:foo", "kernel:bar", "bpf-argument-filtering", "bpf-actlog"})
 }
 
 func (s *backendSuite) TestRequiresSocketcallByNotNeededArch(c *C) {
@@ -643,11 +657,11 @@ func (s *backendSuite) TestRebuildsWithVersionInfoWhenNeeded(c *C) {
 	// change version reported by snap-seccomp
 	snapSeccomp := testutil.MockCommand(c, filepath.Join(dirs.DistroLibExecDir, "snap-seccomp"), `
 if [ "$1" = "version-info" ]; then
-    echo "abcdef 2.3.3 2345abcd"
+    echo "abcdef 2.3.3 2345abcd -"
 fi`)
 	defer snapSeccomp.Restore()
 	updatedProfileHeader := `# snap-seccomp version information:
-# abcdef 2.3.3 2345abcd
+# abcdef 2.3.3 2345abcd -
 `
 	// reload cached version info
 	err = s.Backend.Initialize()
@@ -695,7 +709,7 @@ func (s *backendSuite) TestInitializationDuringBootstrap(c *C) {
 	err := os.MkdirAll(filepath.Dir(snapSeccompInMountedPath), 0755)
 	c.Assert(err, IsNil)
 	snapSeccompInMounted := testutil.MockCommand(c, snapSeccompInMountedPath, `if [ "$1" = "version-info" ]; then
-echo "2345cdef 2.3.4 2345cdef"
+echo "2345cdef 2.3.4 2345cdef -"
 fi`)
 	defer snapSeccompInMounted.Restore()
 
@@ -712,7 +726,7 @@ fi`)
 
 	sb, ok := s.Backend.(*seccomp.Backend)
 	c.Assert(ok, Equals, true)
-	c.Check(sb.VersionInfo(), Equals, "2345cdef 2.3.4 2345cdef")
+	c.Check(sb.VersionInfo(), Equals, seccomp_compiler.VersionInfo("2345cdef 2.3.4 2345cdef -"))
 }
 
 func (s *backendSuite) TestCompilerInitUnhappy(c *C) {
@@ -724,3 +738,83 @@ func (s *backendSuite) TestCompilerInitUnhappy(c *C) {
 	err := s.Backend.Initialize()
 	c.Assert(err, ErrorMatches, "cannot initialize seccomp profile compiler: failed")
 }
+
+func (s *backendSuite) TestSystemUsernamesPolicy(c *C) {
+	snapYaml := `
+name: app
+version: 0.1
+system-usernames:
+  testid: shared
+  testid2: shared
+apps:
+  cmd:
+`
+	snapInfo := snaptest.MockInfo(c, snapYaml, nil)
+	// NOTE: we don't call seccomp.MockTemplate()
+	err := s.Backend.Setup(snapInfo, interfaces.ConfinementOptions{}, s.Repo, s.meas)
+	c.Assert(err, IsNil)
+	// NOTE: we don't call seccomp.MockTemplate()
+	profile := filepath.Join(dirs.SnapSeccompDir, "snap.app.cmd")
+	data, err := ioutil.ReadFile(profile + ".src")
+	c.Assert(err, IsNil)
+	for _, line := range []string{
+		// NOTE: a few randomly picked lines from the real
+		// profile.  Comments and empty lines are avoided as
+		// those can be discarded in the future.
+		"\n# - create_module, init_module, finit_module, delete_module (kernel modules)\n",
+		"\nopen\n",
+		"\ngetuid\n",
+		"\nsetgroups 0 0\n",
+		// and a few randomly picked lines from root syscalls
+		// with extra \n checks to ensure we have the right
+		// "paragraphs" in the generated output
+		"\n\n# allow setresgid to root\n",
+		"\n# allow setresuid to root\n",
+		"\nsetresuid u:root u:root u:root\n",
+		// and a few randomly picked lines from global id syscalls
+		"\n\n# allow setresgid to testid\n",
+		"\n\n# allow setresuid to testid\n",
+		"\nsetresuid -1 u:testid -1\n",
+		// also for the second user
+		"\n\n# allow setresgid to testid2\n",
+		"\n# allow setresuid to testid2\n",
+		"\nsetresuid -1 u:testid2 -1\n",
+	} {
+		c.Assert(string(data), testutil.Contains, line)
+	}
+
+	// make sure the bare syscalls aren't present
+	c.Assert(string(data), Not(testutil.Contains), "setresuid\n")
+}
+
+func (s *backendSuite) TestNoSystemUsernamesPolicy(c *C) {
+	snapYaml := `
+name: app
+version: 0.1
+apps:
+  cmd:
+`
+	snapInfo := snaptest.MockInfo(c, snapYaml, nil)
+	// NOTE: we don't call seccomp.MockTemplate()
+	err := s.Backend.Setup(snapInfo, interfaces.ConfinementOptions{}, s.Repo, s.meas)
+	c.Assert(err, IsNil)
+	// NOTE: we don't call seccomp.MockTemplate()
+	profile := filepath.Join(dirs.SnapSeccompDir, "snap.app.cmd")
+	data, err := ioutil.ReadFile(profile + ".src")
+	c.Assert(err, IsNil)
+	for _, line := range []string{
+		// and a few randomly picked lines from root syscalls
+		"# allow setresgid to root\n",
+		"# allow setresuid to root\n",
+		"setresuid u:root u:root u:root\n",
+		// and a few randomly picked lines from global id syscalls
+		"# allow setresgid to testid\n",
+		"# allow setresuid to testid\n",
+		"setresuid -1 u:testid -1\n",
+	} {
+		c.Assert(string(data), Not(testutil.Contains), line)
+	}
+
+	// make sure the bare syscalls are present
+	c.Assert(string(data), testutil.Contains, "setresuid\n")
+}
