@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2017 Canonical Ltd
+ * Copyright (C) 2016-2019 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -44,20 +44,33 @@ func Add(s *state.State, a asserts.Assertion) error {
 
 // Batch allows to accumulate a set of assertions possibly out of prerequisite order and then add them in one go to the system assertion database.
 type Batch struct {
-	bs   asserts.Backstore
-	refs []*asserts.Ref
+	bs         asserts.Backstore
+	refs       []*asserts.Ref
+	linearized []asserts.Assertion
 }
 
 // NewBatch creates a new Batch to accumulate assertions to add in one go to the system assertion database.
 func NewBatch() *Batch {
 	return &Batch{
-		bs:   asserts.NewMemoryBackstore(),
-		refs: nil,
+		bs:         asserts.NewMemoryBackstore(),
+		refs:       nil,
+		linearized: nil,
 	}
 }
 
+func (b *Batch) committing() error {
+	if b.linearized != nil {
+		return fmt.Errorf("internal error: cannot add to Batch while committing")
+	}
+	return nil
+}
+
 // Add one assertion to the batch.
 func (b *Batch) Add(a asserts.Assertion) error {
+	if err := b.committing(); err != nil {
+		return err
+	}
+
 	if !a.SupportedFormat() {
 		return &asserts.UnsupportedFormatError{Ref: a.Ref(), Format: a.Format()}
 	}
@@ -77,6 +90,10 @@ func (b *Batch) Add(a asserts.Assertion) error {
 // AddStream adds a stream of assertions to the batch.
 // Returns references to to the assertions effectively added.
 func (b *Batch) AddStream(r io.Reader) ([]*asserts.Ref, error) {
+	if err := b.committing(); err != nil {
+		return nil, err
+	}
+
 	start := len(b.refs)
 	dec := asserts.NewDecoder(r)
 	for {
@@ -100,9 +117,22 @@ func (b *Batch) AddStream(r io.Reader) ([]*asserts.Ref, error) {
 	return refs, nil
 }
 
-// Commit adds the batch of assertions to the system assertion database.
-func (b *Batch) Commit(st *state.State) error {
-	db := cachedDB(st)
+func (b *Batch) commitTo(db *asserts.Database) error {
+	if err := b.linearize(db); err != nil {
+		return err
+	}
+
+	// TODO: trigger w. caller a global sanity check if something is revoked
+	// (but try to save as much possible still),
+	// or err is a check error
+	return commitTo(db, b.linearized)
+}
+
+func (b *Batch) linearize(db *asserts.Database) error {
+	if b.linearized != nil {
+		return nil
+	}
+
 	retrieve := func(ref *asserts.Ref) (asserts.Assertion, error) {
 		a, err := b.bs.Get(ref.Type, ref.PrimaryKey, ref.Type.MaxSupportedFormat())
 		if asserts.IsNotFound(err) {
@@ -115,18 +145,31 @@ func (b *Batch) Commit(st *state.State) error {
 		return a, nil
 	}
 
-	// linearize using fetcher
-	f := newFetcher(st, retrieve)
+	// linearize using accumFetcher
+	f := newAccumFetcher(db, retrieve)
 	for _, ref := range b.refs {
 		if err := f.Fetch(ref); err != nil {
 			return err
 		}
 	}
 
-	// TODO: trigger w. caller a global sanity check if something is revoked
-	// (but try to save as much possible still),
-	// or err is a check error
-	return f.commit()
+	b.linearized = f.fetched
+	return nil
+}
+
+// Commit adds the batch of assertions to the system assertion database.
+func (b *Batch) Commit(st *state.State) error {
+	db := cachedDB(st)
+
+	return b.commitTo(db)
+}
+
+// Precheck pre-checks whether adding the batch of assertions to the system assertion database should fully succeed.
+func (b *Batch) Precheck(st *state.State) error {
+	db := cachedDB(st)
+	db = db.WithStackedBackstore(asserts.NewMemoryBackstore())
+
+	return b.commitTo(db)
 }
 
 func findError(format string, ref *asserts.Ref, err error) error {
@@ -139,10 +182,11 @@ func findError(format string, ref *asserts.Ref, err error) error {
 
 // RefreshSnapDeclarations refetches all the current snap declarations and their prerequisites.
 func RefreshSnapDeclarations(s *state.State, userID int) error {
-	modelAs, err := snapstate.ModelPastSeeding(s)
+	deviceCtx, err := snapstate.DevicePastSeeding(s, nil)
 	if err != nil {
 		return err
 	}
+	modelAs := deviceCtx.Model()
 
 	snapStates, err := snapstate.All(s)
 	if err != nil {
@@ -175,7 +219,7 @@ func RefreshSnapDeclarations(s *state.State, userID int) error {
 
 		return nil
 	}
-	return doFetch(s, userID, fetching)
+	return doFetch(s, userID, deviceCtx, fetching)
 }
 
 type refreshControlError struct {
@@ -198,7 +242,7 @@ func (e *refreshControlError) Error() string {
 // it returns a validated subset in validated and a summary error if not all
 // candidates validated. ignoreValidation is a set of snap-instance-names that
 // should not be gated.
-func ValidateRefreshes(s *state.State, snapInfos []*snap.Info, ignoreValidation map[string]bool, userID int) (validated []*snap.Info, err error) {
+func ValidateRefreshes(s *state.State, snapInfos []*snap.Info, ignoreValidation map[string]bool, userID int, deviceCtx snapstate.DeviceContext) (validated []*snap.Info, err error) {
 	// maps gated snap-ids to gating snap-ids
 	controlled := make(map[string][]string)
 	// maps gating snap-ids to their snap names
@@ -271,7 +315,7 @@ func ValidateRefreshes(s *state.State, snapInfos []*snap.Info, ignoreValidation
 			}
 			return nil
 		}
-		err := doFetch(s, userID, fetching)
+		err := doFetch(s, userID, deviceCtx, fetching)
 		if err != nil {
 			errs = append(errs, fmt.Errorf("cannot refresh %q to revision %s: %v", candInfo.InstanceName(), candInfo.Revision, err))
 			continue
