@@ -24,6 +24,8 @@ import (
 
 	"github.com/snapcore/snapd/overlord/configstate"
 	"github.com/snapcore/snapd/overlord/hookstate"
+	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/storecontext"
 	"github.com/snapcore/snapd/store"
 )
 
@@ -59,6 +61,11 @@ func (o *Overlord) Engine() *StateEngine {
 	return o.stateEng
 }
 
+// NewStore exposes newStore.
+func (o *Overlord) NewStore(devBE storecontext.DeviceBackend) snapstate.StoreService {
+	return o.newStore(devBE)
+}
+
 // MockStoreNew mocks store.New as called by overlord.New.
 func MockStoreNew(new func(*store.Config, store.DeviceAndAuthContext) *store.Store) (restore func()) {
 	storeNew = new
