@@ -198,3 +198,18 @@ hooks:
 	c.Assert(err, ErrorMatches, `cannot rename slot "old" to "new": no such slot`)
 
 }
+
+func (s *snapTestSuite) TestMockSnapWithFiles(c *C) {
+	snapInfo := snaptest.MockSnapWithFiles(c, sampleYaml, &snap.SideInfo{Revision: snap.R(42)}, [][]string{
+		{"foo/bar", "bar"},
+		{"bar", "not foo"},
+		{"meta/gadget.yaml", "gadget yaml\nmulti line"},
+	})
+	// Data from YAML is used
+	c.Check(snapInfo.InstanceName(), Equals, "sample")
+	// Data from SideInfo is used
+	c.Check(snapInfo.Revision, Equals, snap.R(42))
+	c.Check(filepath.Join(snapInfo.MountDir(), "bar"), testutil.FileEquals, "not foo")
+	c.Check(filepath.Join(snapInfo.MountDir(), "foo/bar"), testutil.FileEquals, "bar")
+	c.Check(filepath.Join(snapInfo.MountDir(), "meta/gadget.yaml"), testutil.FileEquals, "gadget yaml\nmulti line")
+}
