@@ -218,6 +218,7 @@ func (s *ValidateSuite) TestValidateAppSocketsValidListenStreamAddresses(c *C) {
 		// socket paths using variables as prefix
 		"$SNAP_DATA/my.socket",
 		"$SNAP_COMMON/my.socket",
+		"$XDG_RUNTIME_DIR/my.socket",
 		// abstract sockets
 		"@snap.mysnap.my.socket",
 		// addresses and ports
@@ -273,7 +274,7 @@ func (s *ValidateSuite) TestValidateAppSocketsInvalidListenStreamPathPrefix(c *C
 		err := ValidateApp(app)
 		c.Assert(
 			err, ErrorMatches,
-			`invalid definition of socket "sock": invalid "listen-stream": must have a prefix of \$SNAP_DATA or \$SNAP_COMMON`)
+			`invalid definition of socket "sock": invalid "listen-stream": must have a prefix of \$SNAP_DATA, \$SNAP_COMMON or \$XDG_RUNTIME_DIR`)
 	}
 }
 
@@ -572,7 +573,8 @@ version: 1
 plugs:
   p--lug: null
 `
-	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml1), nil)
+	strk := NewScopedTracker()
+	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml1), nil, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Plugs, HasLen, 1)
 	err = Validate(info)
@@ -584,7 +586,8 @@ version: 1
 slots:
   s--lot: null
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml2), nil)
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml2), nil, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Slots, HasLen, 1)
 	err = Validate(info)
@@ -597,7 +600,8 @@ plugs:
   plug:
     interface: i--face
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml3), nil)
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml3), nil, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Plugs, HasLen, 1)
 	err = Validate(info)
@@ -610,13 +614,54 @@ slots:
   slot:
     interface: i--face
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml4), nil)
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml4), nil, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Slots, HasLen, 1)
 	err = Validate(info)
 	c.Assert(err, ErrorMatches, `invalid interface name "i--face" for slot "slot"`)
 }
 
+func (s *ValidateSuite) TestValidateBaseNone(c *C) {
+	const yaml = `name: requires-base
+version: 1
+base: none
+`
+	strk := NewScopedTracker()
+	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), nil, strk)
+	c.Assert(err, IsNil)
+	err = Validate(info)
+	c.Assert(err, IsNil)
+	c.Check(info.Base, Equals, "none")
+}
+
+func (s *ValidateSuite) TestValidateBaseNoneError(c *C) {
+	yamlTemplate := `name: use-base-none
+version: 1
+base: none
+
+%APPS_OR_HOOKS%
+`
+	const apps = `
+apps:
+  useradd:
+    command: bin/true
+`
+	const hooks = `
+hooks:
+  configure:
+`
+
+	for _, appsOrHooks := range []string{apps, hooks} {
+		yaml := strings.Replace(yamlTemplate, "%APPS_OR_HOOKS%", appsOrHooks, -1)
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), nil, strk)
+		c.Assert(err, IsNil)
+		err = Validate(info)
+		c.Assert(err, ErrorMatches, `cannot have apps or hooks with base "none"`)
+	}
+}
+
 type testConstraint string
 
 func (constraint testConstraint) IsOffLimits(path string) bool {
@@ -667,6 +712,8 @@ func (s *ValidateSuite) TestValidateLayout(c *C) {
 		ErrorMatches, `layout "/dev" in an off-limits area`)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/dev/foo", Type: "tmpfs"}, nil),
 		ErrorMatches, `layout "/dev/foo" in an off-limits area`)
+	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/home", Type: "tmpfs"}, nil),
+		ErrorMatches, `layout "/home" in an off-limits area`)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/proc", Type: "tmpfs"}, nil),
 		ErrorMatches, `layout "/proc" in an off-limits area`)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/sys", Type: "tmpfs"}, nil),
@@ -689,10 +736,11 @@ func (s *ValidateSuite) TestValidateLayout(c *C) {
 		ErrorMatches, `layout "/lib/firmware" in an off-limits area`)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/lib/modules", Type: "tmpfs"}, nil),
 		ErrorMatches, `layout "/lib/modules" in an off-limits area`)
+	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/tmp", Type: "tmpfs"}, nil),
+		ErrorMatches, `layout "/tmp" in an off-limits area`)
 
 	// Several valid layouts.
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/foo", Type: "tmpfs", Mode: 01755}, nil), IsNil)
-	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/tmp", Type: "tmpfs"}, nil), IsNil)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/usr", Bind: "$SNAP/usr"}, nil), IsNil)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/var", Bind: "$SNAP_DATA/var"}, nil), IsNil)
 	c.Check(ValidateLayout(&Layout{Snap: si, Path: "/var", Bind: "$SNAP_COMMON/var"}, nil), IsNil)
@@ -726,7 +774,8 @@ layout:
 `
 
 	for _, yaml := range []string{yaml1, yaml1rev} {
-		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)})
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)}, strk)
 		c.Assert(err, IsNil)
 		c.Assert(info.Layout, HasLen, 2)
 		err = ValidateLayoutAll(info)
@@ -751,7 +800,8 @@ layout:
     bind: $SNAP
 `
 	for _, yaml := range []string{yaml2, yaml2rev} {
-		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)})
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)}, strk)
 		c.Assert(err, IsNil)
 		c.Assert(info.Layout, HasLen, 2)
 		err = ValidateLayoutAll(info)
@@ -776,7 +826,8 @@ layout:
     bind: $SNAP_DATA/foo
 `
 	for _, yaml := range []string{yaml3, yaml3rev} {
-		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)})
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)}, strk)
 		c.Assert(err, IsNil)
 		c.Assert(info.Layout, HasLen, 2)
 		err = ValidateLayoutAll(info)
@@ -801,7 +852,8 @@ layout:
     symlink: $SNAP_DATA/foo
 `
 	for _, yaml := range []string{yaml4, yaml4rev} {
-		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)})
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)}, strk)
 		c.Assert(err, IsNil)
 		c.Assert(info.Layout, HasLen, 2)
 		err = ValidateLayoutAll(info)
@@ -826,7 +878,8 @@ layout:
     symlink: $SNAP_DATA/foo
 `
 	for _, yaml := range []string{yaml5, yaml5rev} {
-		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)})
+		strk := NewScopedTracker()
+		info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml), &SideInfo{Revision: R(42)}, strk)
 		c.Assert(err, IsNil)
 		c.Assert(info.Layout, HasLen, 2)
 		err = ValidateLayoutAll(info)
@@ -841,7 +894,8 @@ layout:
   /etc/norf:
     bind-file: $SNAP/etc/norf
 `
-	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml6), &SideInfo{Revision: R(42)})
+	strk := NewScopedTracker()
+	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml6), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 1)
 	err = ValidateLayoutAll(info)
@@ -858,7 +912,9 @@ layout:
   /etc/corge:
     bind-file: $SNAP/etc/norf
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml7), &SideInfo{Revision: R(42)})
+
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml7), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 2)
 	err = ValidateLayoutAll(info)
@@ -873,7 +929,8 @@ layout:
   /etc/corge:
     bind: $SNAP/etc/norf
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml8), &SideInfo{Revision: R(42)})
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml8), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 2)
 	err = ValidateLayoutAll(info)
@@ -888,7 +945,8 @@ layout:
   /etc/corge:
     bind: /snap/clashing-source-path-3/42/etc/norf
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml9), &SideInfo{Revision: R(42)})
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml9), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 2)
 	err = ValidateLayoutAll(info)
@@ -903,7 +961,9 @@ layout:
   /etc/corge:
     symlink: $SNAP/etc/norf
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml10), &SideInfo{Revision: R(42)})
+
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml10), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 2)
 	err = ValidateLayoutAll(info)
@@ -918,11 +978,28 @@ layout:
   /etc/corge:
     symlink: $SNAP/etc/norf
 `
-	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml11), &SideInfo{Revision: R(42)})
+
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml11), &SideInfo{Revision: R(42)}, strk)
 	c.Assert(err, IsNil)
 	c.Assert(info.Layout, HasLen, 2)
 	err = ValidateLayoutAll(info)
 	c.Assert(err, IsNil)
+
+	// Layout replacing files in another snap's mount p oit
+	const yaml12 = `
+name: this-snap
+layout:
+  /snap/that-snap/current/stuff:
+    symlink: $SNAP/stuff
+`
+
+	strk = NewScopedTracker()
+	info, err = InfoFromSnapYamlWithSideInfo([]byte(yaml12), &SideInfo{Revision: R(42)}, strk)
+	c.Assert(err, IsNil)
+	c.Assert(info.Layout, HasLen, 1)
+	err = ValidateLayoutAll(info)
+	c.Assert(err, ErrorMatches, `layout "/snap/that-snap/current/stuff" defines a layout in space belonging to another snap`)
 }
 
 func (s *YamlSuite) TestValidateAppStartupOrder(c *C) {
@@ -1226,6 +1303,38 @@ base: bar
 	c.Check(err, ErrorMatches, `cannot have "base" field on "os" snap "foo"`)
 }
 
+func (s *ValidateSuite) TestValidateOsCanHaveBaseNone(c *C) {
+	info, err := InfoFromSnapYaml([]byte(`name: foo
+version: 1.0
+type: os
+base: none
+`))
+	c.Assert(err, IsNil)
+	c.Assert(Validate(info), IsNil)
+}
+
+func (s *ValidateSuite) TestValidateBaseInorrectSnapName(c *C) {
+	info, err := InfoFromSnapYaml([]byte(`name: foo
+version: 1.0
+base: aAAAA
+`))
+	c.Assert(err, IsNil)
+
+	err = Validate(info)
+	c.Check(err, ErrorMatches, `invalid base name: invalid snap name: \"aAAAA\"`)
+}
+
+func (s *ValidateSuite) TestValidateBaseSnapInstanceNameNotAllowed(c *C) {
+	info, err := InfoFromSnapYaml([]byte(`name: foo
+version: 1.0
+base: foo_abc
+`))
+	c.Assert(err, IsNil)
+
+	err = Validate(info)
+	c.Check(err, ErrorMatches, `base cannot specify a snap instance name: "foo_abc"`)
+}
+
 func (s *ValidateSuite) TestValidateBaseCannotHaveBase(c *C) {
 	info, err := InfoFromSnapYaml([]byte(`name: foo
 version: 1.0
@@ -1238,6 +1347,16 @@ base: bar
 	c.Check(err, ErrorMatches, `cannot have "base" field on "base" snap "foo"`)
 }
 
+func (s *ValidateSuite) TestValidateBaseCanHaveBaseNone(c *C) {
+	info, err := InfoFromSnapYaml([]byte(`name: foo
+version: 1.0
+type: base
+base: none
+`))
+	c.Assert(err, IsNil)
+	c.Assert(Validate(info), IsNil)
+}
+
 func (s *ValidateSuite) TestValidateCommonIDs(c *C) {
 	meta := `
 name: foo
@@ -1440,3 +1559,18 @@ apps:
 		}
 	}
 }
+
+func (s *ValidateSuite) TestValidateSystemUsernames(c *C) {
+	const yaml1 = `name: binary
+version: 1.0
+system-usernames:
+  "b@d": shared
+`
+
+	strk := NewScopedTracker()
+	info, err := InfoFromSnapYamlWithSideInfo([]byte(yaml1), nil, strk)
+	c.Assert(err, IsNil)
+	c.Assert(info.SystemUsernames, HasLen, 1)
+	err = Validate(info)
+	c.Assert(err, ErrorMatches, `invalid system username "b@d"`)
+}
