@@ -441,11 +441,22 @@ distro_install_build_snapd(){
         apt install -y --only-upgrade snapd
         mv sources.list.back /etc/apt/sources.list
         apt update
+
         # On trusty we may pull in a new hwe-kernel that is needed to run the
         # snapd tests. We need to reboot to actually run this kernel.
         if [[ "$SPREAD_SYSTEM" = ubuntu-14.04-* ]] && [ "$SPREAD_REBOOT" = 0 ]; then
             REBOOT
         fi
+    elif [ -n "$PPA_VALIDATION_NAME" ]; then
+        apt install -y snapd
+        add-apt-repository -y "$PPA_VALIDATION_NAME"
+        apt update
+        apt install -y --only-upgrade snapd
+        add-apt-repository --remove "$PPA_VALIDATION_NAME"
+        apt update
+
+        # Double check that it really comes from the PPA
+        apt show snapd | grep "APT-Sources: http.*ppa.launchpad.net"
     else
         packages=
         case "$SPREAD_SYSTEM" in
@@ -478,8 +489,7 @@ distro_install_build_snapd(){
                 # We need to wait until the man db cache is updated before do daemon-reexec
                 # Otherwise the service fails and the system will be degraded during tests executions
                 for i in $(seq 20); do
-                    man_db_cache_update_service=$(systemctl list-units | grep -E 'run-.*.service' | awk '{print $1;}')
-                    if [ -z "$man_db_cache_update_service" ]; then
+                    if ! systemctl is-active run-*.service; then
                         break
                     fi
                     sleep .5
@@ -591,6 +601,7 @@ pkg_dependencies_ubuntu_classic(){
                 gccgo-6
                 evolution-data-server
                 gnome-online-accounts
+                packagekit
                 "
                 pkg_linux_image_extra
             ;;
@@ -601,24 +612,24 @@ pkg_dependencies_ubuntu_classic(){
                 gnome-online-accounts
                 kpartx
                 libvirt-bin
+                packagekit
                 qemu
                 x11-utils
                 xvfb
                 "
                 pkg_linux_image_extra
             ;;
-        ubuntu-17.10-64)
-                pkg_linux_image_extra
-            ;;
         ubuntu-18.04-64)
             echo "
                 gccgo-8
                 evolution-data-server
+                packagekit
                 "
             ;;
-        ubuntu-18.10-64)
+        ubuntu-19.04-64)
             echo "
                 evolution-data-server
+                packagekit
                 "
             ;;
         ubuntu-*)
@@ -631,6 +642,7 @@ pkg_dependencies_ubuntu_classic(){
                 eatmydata
                 evolution-data-server
                 net-tools
+                packagekit
                 sbuild
                 "
             ;;
@@ -673,6 +685,7 @@ pkg_dependencies_fedora(){
         mock
         net-tools
         nfs-utils
+        PackageKit
         python3-yaml
         python3-dbus
         python3-gobject
@@ -701,6 +714,7 @@ pkg_dependencies_amazon(){
         nc
         net-tools
         nfs-utils
+        PackageKit
         system-lsb-core
         rpm-build
         xdg-user-dirs
@@ -723,6 +737,7 @@ pkg_dependencies_opensuse(){
         lsb-release
         man
         nfs-kernel-server
+        PackageKit
         python3-yaml
         netcat-openbsd
         osc
@@ -752,6 +767,7 @@ pkg_dependencies_arch(){
     net-tools
     nfs-utils
     openbsd-netcat
+    packagekit
     python
     python-docutils
     python-dbus
