@@ -32,10 +32,12 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/sys"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/timeout"
 	"github.com/snapcore/snapd/timeutil"
+	"github.com/snapcore/snapd/timings"
 )
 
 type interacter interface {
@@ -100,8 +102,8 @@ func stopService(sysd systemd.Systemd, app *snap.AppInfo, inter interacter) erro
 // StartServices starts service units for the applications from the snap which
 // are services. Service units will be started in the order provided by the
 // caller.
-func StartServices(apps []*snap.AppInfo, inter interacter) (err error) {
-	sysd := systemd.New(dirs.GlobalRootDir, inter)
+func StartServices(apps []*snap.AppInfo, inter interacter, tm timings.Measurer) (err error) {
+	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
 
 	services := make([]string, 0, len(apps))
 	for _, app := range apps {
@@ -153,7 +155,10 @@ func StartServices(apps []*snap.AppInfo, inter interacter) (err error) {
 				return err
 			}
 
-			if err := sysd.Start(socketService); err != nil {
+			timings.Run(tm, "start-socket-service", fmt.Sprintf("start socket service %q", socketService), func(nested timings.Measurer) {
+				err = sysd.Start(socketService)
+			})
+			if err != nil {
 				return err
 			}
 		}
@@ -165,7 +170,10 @@ func StartServices(apps []*snap.AppInfo, inter interacter) (err error) {
 				return err
 			}
 
-			if err := sysd.Start(timerService); err != nil {
+			timings.Run(tm, "start-timer-service", fmt.Sprintf("start timer service %q", timerService), func(nested timings.Measurer) {
+				err = sysd.Start(timerService)
+			})
+			if err != nil {
 				return err
 			}
 		}
@@ -178,7 +186,10 @@ func StartServices(apps []*snap.AppInfo, inter interacter) (err error) {
 		// by one, see:
 		// https://github.com/systemd/systemd/issues/8102
 		// https://lists.freedesktop.org/archives/systemd-devel/2018-January/040152.html
-		if err := sysd.Start(srv); err != nil {
+		timings.Run(tm, "start-service", fmt.Sprintf("start service %q", srv), func(nested timings.Measurer) {
+			err = sysd.Start(srv)
+		})
+		if err != nil {
 			// cleanup was set up by iterating over apps
 			return err
 		}
@@ -189,11 +200,11 @@ func StartServices(apps []*snap.AppInfo, inter interacter) (err error) {
 
 // AddSnapServices adds service units for the applications from the snap which are services.
 func AddSnapServices(s *snap.Info, inter interacter) (err error) {
-	if s.SnapName() == "snapd" {
+	if s.GetType() == snap.TypeSnapd {
 		return writeSnapdServicesOnCore(s, inter)
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, inter)
+	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
 	var written []string
 	var enabled []string
 	defer func() {
@@ -282,8 +293,8 @@ func AddSnapServices(s *snap.Info, inter interacter) (err error) {
 }
 
 // StopServices stops service units for the applications from the snap which are services.
-func StopServices(apps []*snap.AppInfo, reason snap.ServiceStopReason, inter interacter) error {
-	sysd := systemd.New(dirs.GlobalRootDir, inter)
+func StopServices(apps []*snap.AppInfo, reason snap.ServiceStopReason, inter interacter, tm timings.Measurer) error {
+	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
 
 	logger.Debugf("StopServices called for %q, reason: %v", apps, reason)
 	for _, app := range apps {
@@ -302,7 +313,12 @@ func StopServices(apps []*snap.AppInfo, reason snap.ServiceStopReason, inter int
 				continue
 			}
 		}
-		if err := stopService(sysd, app, inter); err != nil {
+
+		var err error
+		timings.Run(tm, "stop-service", fmt.Sprintf("stop service %q", app.ServiceName()), func(nested timings.Measurer) {
+			err = stopService(sysd, app, inter)
+		})
+		if err != nil {
 			return err
 		}
 
@@ -324,7 +340,7 @@ func StopServices(apps []*snap.AppInfo, reason snap.ServiceStopReason, inter int
 
 // RemoveSnapServices disables and removes service units for the applications from the snap which are services.
 func RemoveSnapServices(s *snap.Info, inter interacter) error {
-	sysd := systemd.New(dirs.GlobalRootDir, inter)
+	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
 	nservices := 0
 
 	for _, app := range s.Apps {
@@ -585,6 +601,11 @@ func generateSnapSocketFiles(app *snap.AppInfo) (*map[string][]byte, error) {
 func renderListenStream(socket *snap.SocketInfo) string {
 	snap := socket.App.Snap
 	listenStream := strings.Replace(socket.ListenStream, "$SNAP_DATA", snap.DataDir(), -1)
+	// TODO: when we support User/Group in the generated systemd unit,
+	// adjust this accordingly
+	serviceUserUid := sys.UserID(0)
+	runtimeDir := snap.UserXdgRuntimeDir(serviceUserUid)
+	listenStream = strings.Replace(listenStream, "$XDG_RUNTIME_DIR", runtimeDir, -1)
 	return strings.Replace(listenStream, "$SNAP_COMMON", snap.CommonDataDir(), -1)
 }
 
